﻿ #region Disclaimer/Info
 
 /////////////////////////////////////////////////////////////////////////////////////////////////
 //
 //   File:		Comment.Builder.cs
 //   Website:		http://dexterblogengine.com/
 //   Authors:		http://dexterblogengine.com/About.ashx
 //   Rev:		1
 //   Created:		19/01/2011
 //   Last edit:		19/01/2011
 //   License:		GNU Library General Public License (LGPL)
 //   File:            Comment.Builder.cs
 //   For updated news and information please visit http://dexterblogengine.com/
 //   Dexter is hosted to Codeplex at http://dexterblogengine.codeplex.com
 //   For any question contact info@dexterblogengine.com
 //
 ///////////////////////////////////////////////////////////////////////////////////////////////////
 
 #endregion
 
using System;
using System.Net.Mail;
using Topics.Radical.Validation;

namespace Dexter.Domain.Model {
	public partial class Comment {

		/// <summary>
		/// Creates a new instance of <see cref="Comment"/>.
		/// </summary>
		/// <param name="item">The item <see cref="Item"/>.</param>
		/// <param name="website">The website of the <see cref="Comment"/> author.</param>
		/// <param name="email">The email of the <see cref="Comment"/> author.</param>
		/// <param name="message">The message of the <see cref="Comment"/> </param>
		/// <param name="name">The name of the <see cref="Comment"/> author.</param>
		/// <param name="parent">The parent <see cref="Comment"/>.</param>
		/// <returns>A new instance of <see cref="Comment"/></returns>
		/// <exception cref="ArgumentNullException">If <paramref name="item"/> is <c>null</c>.</exception>
		/// <exception cref="ArgumentNullException">If <paramref name="email"/> is <c>null</c>.</exception>
		/// <exception cref="ArgumentNullException">If <paramref name="message"/> is <c>null</c> or <c>empty</c>.</exception>
		/// <exception cref="ArgumentNullException">If <paramref name="name"/> is <c>null</c> or <c>empty</c>.</exception>
		/// <exception cref="ArgumentOutOfRangeException">Will be throw if the the parent <see cref="Comment"/> is not null but the property id of its will be lesser then 1.</exception>
		public static Comment CreateNewComment(Item item, Uri website, MailAddress email, string message, string name, Comment parent)
		{
			Ensure.That ( item )
				.Named ( "item" )
				.WithPreview ( ( v , e ) => Logger.Warn ( v.GetFullErrorMessage ( ) , e ) )
				.IsNotNull ( )
				.WithMessage ( "Invalid Item." );

			Ensure.That ( email )
				.Named ( "email" )
				.WithPreview ( ( v , e ) => Logger.Warn ( v.GetFullErrorMessage ( ) , e ) )
				.IsNotNull()
				.WithMessage ( "Invalid email." );

			Ensure.That ( message )
				.Named ( "message" )
				.WithPreview ( ( v , e ) => Logger.Warn ( v.GetFullErrorMessage ( ) , e ) )
				.IsNotNullNorEmpty ( )
				.WithMessage ( "Invalid message." );

			Ensure.That ( name )
				.Named ( "name" )
				.WithPreview ( ( v , e ) => Logger.Warn ( v.GetFullErrorMessage ( ) , e ) )
				.IsNotNullNorEmpty ( );

			if ( parent != null ) {
				Ensure.That ( parent.Id )
					.Named ( "parent.Id" )
					.WithPreview ( ( v , e ) => Logger.Warn ( v.GetFullErrorMessage ( ) , e ) )
					.IsGreaterThen ( 0 )
					.WithMessage ( "The parent category Key is invalid." );
			}

			return new Comment {
				Item = item ,
				WebSite = website ,
				Email = email ,
				Message = message ,
				Name = name ,
				Parent = parent
			};
		}
	}
}
