﻿ #region Disclaimer/Info
 
 /////////////////////////////////////////////////////////////////////////////////////////////////
 //
 //   File:		Page.Builder.cs
 //   Website:		http://dexterblogengine.com/
 //   Authors:		http://dexterblogengine.com/About.ashx
 //   Rev:		1
 //   Created:		19/01/2011
 //   Last edit:		19/01/2011
 //   License:		GNU Library General Public License (LGPL)
 //   File:            Page.Builder.cs
 //   For updated news and information please visit http://dexterblogengine.com/
 //   Dexter is hosted to Codeplex at http://dexterblogengine.codeplex.com
 //   For any question contact info@dexterblogengine.com
 //
 ///////////////////////////////////////////////////////////////////////////////////////////////////
 
 #endregion
 
using System;
using Topics.Radical.Validation;

namespace Dexter.Domain.Model {
	public partial class Page {

		/// <summary>
		/// Creates a new instance of <see cref="Page"/>.
		/// </summary>
		/// <param name="title">The title of the page.</param>
		/// <param name="body">The html of the page.</param>
		/// <returns>A new instance of <see cref="Page"/>.</returns>
		/// <exception cref="ArgumentNullException">If <paramref name="title"/> is <c>null</c> or <c>empty</c>.</exception>
		/// <exception cref="ArgumentNullException">If <paramref name="body"/> is <c>null</c> or <c>empty</c>.</exception>
		public static Page CreateNewPage ( string title , string body ) {
			return CreateNewPage ( title , body , null );
		}

		/// <summary>
		/// Creates a new instance of <see cref="Page"/>.
		/// </summary>
		/// <param name="title">The title of the page.</param>
		/// <param name="body">The html of the page.</param>
		/// <param name="slug">The slug (optional).</param>
		/// <returns>A new instance of <see cref="Page"/>.</returns>
		/// <exception cref="ArgumentNullException">If <paramref name="title"/> is <c>null</c> or <c>empty</c>.</exception>
		/// <exception cref="ArgumentNullException">If <paramref name="body"/> is <c>null</c> or <c>empty</c>.</exception>
		public static Page CreateNewPage ( string title , string body , string slug ) {
			return CreateNewPage ( title , body , slug , null );
		}

		/// <summary>
		/// Creates a new instance of <see cref="Page"/>.
		/// </summary>
		/// <param name="title">The title of the page.</param>
		/// <param name="body">The html of the page.</param>
		/// <param name="slug">The slug (optional).</param>
		/// <param name="parent">The parent page (option).</param>
		/// <returns>A new instance of <see cref="Page"/>.</returns>
		/// <exception cref="ArgumentNullException">If <paramref name="title"/> is <c>null</c> or <c>empty</c>.</exception>
		/// <exception cref="ArgumentNullException">If <paramref name="body"/> is <c>null</c> or <c>empty</c>.</exception>
		/// <exception cref="ArgumentOutOfRangeException">Will be throw if the the parent <see cref="Page"/> is not null but the property id of its will be lesser then 1.</exception>
		public static Page CreateNewPage ( string title , string body , string slug , Page parent ) {
			Ensure.That ( title )
				.Named ( "name" )
				.WithPreview ( ( v , e ) => Logger.Warn ( v.GetFullErrorMessage ( ) , e ) )
				.IsNotNullNorEmpty ( )
				.WithMessage ( "Invalid page title" );

			Ensure.That ( body )
				.Named ( "body" )
				.WithPreview ( ( v , e ) => Logger.Warn ( v.GetFullErrorMessage ( ) , e ) )
				.IsNotNullNorEmpty ( )
				.WithMessage ( "Invalid page body" );

			if ( parent != null ) {
				Ensure.That ( parent.Id )
					.Named ( "parent.Id" )
					.WithPreview ( ( v , e ) => Logger.Warn ( v.GetFullErrorMessage ( ) , e ) )
					.IsGreaterThen ( 0 )
					.WithMessage ( "The parent page key is invalid." );
			}

			return new Page {
				Title = title ,
				FormattedBody = body ,
				Slug = slug ,
				Parent = parent
			};
		}
	}
}
