﻿ #region Disclaimer/Info
 
 /////////////////////////////////////////////////////////////////////////////////////////////////
 //
 //   File:		TagItem.Builder.cs
 //   Website:		http://dexterblogengine.com/
 //   Authors:		http://dexterblogengine.com/About.ashx
 //   Rev:		1
 //   Created:		19/01/2011
 //   Last edit:		19/01/2011
 //   License:		GNU Library General Public License (LGPL)
 //   File:            TagItem.Builder.cs
 //   For updated news and information please visit http://dexterblogengine.com/
 //   Dexter is hosted to Codeplex at http://dexterblogengine.codeplex.com
 //   For any question contact info@dexterblogengine.com
 //
 ///////////////////////////////////////////////////////////////////////////////////////////////////
 
 #endregion
 

using System;
using Topics.Radical.Validation;

namespace Dexter.Domain.Model {
	public partial class TagItem {
		/// <summary>
		/// Creates a new instance of <see cref="TagItem"/>.
		/// </summary>
		/// <param name="name">The name of the Blog Roll.</param>
		/// <param name="item">The <see cref="Item"/> of the tag..</param>
		/// <returns>A new instance of <see cref="BlogRoll"/></returns>
		/// <exception cref="ArgumentNullException"> If <paramref name="name"/> is <c>null</c> or <c>empty</c>.</exception>
		/// <exception cref="ArgumentNullException"> If <paramref name="item"/> is <c>null</c>.</exception>
		public static TagItem CreateNewTagItem ( string name, Item item ) {
			Ensure.That ( name )
			.Named ( "name" )
			.WithPreview ( ( v, e ) => Logger.Warn ( v.GetFullErrorMessage ( ), e ) )
			.IsNotNullNorEmpty ( )
			.WithMessage ( "Invalid tag name." );
			
			Ensure.That ( item )
				.Named ( "item" )
				.WithPreview ( ( v, e ) => Logger.Warn ( v.GetFullErrorMessage ( ), e ) )
				.IsNotNull ( )
				.WithMessage ( "Invalid tag item." );

			return new TagItem ( name , item );
		}
	}
}
