﻿using System;
using System.Collections.Generic;
using System.Reflection;
using ConfOrm;
using ConfOrm.NH;
using ConfOrm.Patterns;
using ConfOrm.Shop.CoolNaming;
using ConfOrm.Shop.NamingAppliers;
using ConfOrm.Shop.Subclassing;
using NHibernate.Mapping.ByCode;

namespace Dexter.Repository.NHibernate.ConfORM {
	public class DexterPatternsAppliersHolder : EmptyPatternsAppliersHolder {
		public DexterPatternsAppliersHolder ( IDomainInspector domainInspector ) {
			if ( domainInspector == null ) {
				throw new ArgumentNullException ( "domainInspector" );
			}

			rootClass = new List <IPatternApplier <Type , IClassAttributesMapper>> ( ) {
				new DiscriminatorColumnNameApplier ( domainInspector ) ,
				new DiscriminatorIndexNameApplier ( domainInspector ) ,
				new ClassDiscriminatorValueAsClassNameApplier ( domainInspector )
			};

			base.joinedSubclass = new List <IPatternApplier <Type , IJoinedSubclassAttributesMapper>> {
				new JoinedSubclassOnDeleteApplier ( ) ,
				new JoinedSubclassKeyAsRootIdColumnApplier ( domainInspector )
			};
			base.subclass = new List <IPatternApplier <Type , ISubclassAttributesMapper>> ( );

			base.unionSubclass = new List <IPatternApplier <Type , IUnionSubclassAttributesMapper>> ( );

			base.poid = new List <IPatternApplier <MemberInfo , IIdMapper>> {
				new MemberNoSetterToFieldAccessorApplier <IIdMapper> ( ) ,
				new NoPoidGuidApplier ( ) ,
				new BidirectionalOneToOneAssociationPoidApplier ( domainInspector )
			};

			base.property = new List <IPatternApplier <MemberInfo , IPropertyMapper>> {
				new MemberReadOnlyAccessorApplier <IPropertyMapper> ( ) ,
				new MemberNoSetterToFieldAccessorApplier <IPropertyMapper> ( ) ,
				new MemberToFieldAccessorApplier <IPropertyMapper> ( )
			};

			base.propertyPath = new List <IPatternApplier <PropertyPath , IPropertyMapper>> {
				new ComponentPropertyColumnNameApplier ( )
			};

			base.collection = new List <IPatternApplier <MemberInfo , ICollectionPropertiesMapper>> {
				new MemberReadOnlyAccessorApplier <ICollectionPropertiesMapper> ( ) ,
				new MemberNoSetterToFieldAccessorApplier <ICollectionPropertiesMapper> ( ) ,
				new MemberToFieldAccessorApplier <ICollectionPropertiesMapper> ( ) ,
				new BidirectionalOneToManyCascadeApplier ( domainInspector ) ,
				new BidirectionalOneToManyInverseApplier ( domainInspector ) ,
				new BidirectionalOneToManyOnDeleteConstraintApplier ( domainInspector ) ,
				new BidirectionalManyToManyInverseApplier ( domainInspector ) ,
				new PolymorphismBidirectionalOneToManyInverseApplier ( domainInspector ) ,
				new PolymorphismBidirectionalOneToManyCascadeApplier ( domainInspector ) ,
				new PolymorphismBidirectionalOneToManyOnDeleteConstraintApplier ( domainInspector ) ,
				new PolymorphismBidirectionalAnyToManyKeyColumnApplier ( domainInspector ) ,
				new PolymorphismBidirectionalAnyToManyWhereApplier ( domainInspector )
			};

			base.collectionPath = new List <IPatternApplier <PropertyPath , ICollectionPropertiesMapper>> {
				new ManyToManyInCollectionTableApplier ( domainInspector ) ,
				new ManyToManyKeyIdColumnApplier ( domainInspector ) ,
				new OneToManyKeyColumnApplier ( domainInspector ) ,
				new CollectionOfElementsTableApplier ( domainInspector ) ,
				new CollectionOfElementsKeyColumnApplier ( domainInspector ) ,
				new CollectionOfComponentsTableApplier ( domainInspector ) ,
				new CollectionOfComponentsKeyColumnApplier ( domainInspector )
			};

			base.listPath = new List <IPatternApplier <PropertyPath , IListPropertiesMapper>> {
				new ListIndexAsPropertyPosColumnNameApplier ( )
			};

			base.manyToOne = new List <IPatternApplier <MemberInfo , IManyToOneMapper>> {
				new MemberReadOnlyAccessorApplier <IManyToOneMapper> ( ) ,
				new MemberNoSetterToFieldAccessorApplier <IManyToOneMapper> ( ) ,
				new MemberToFieldAccessorApplier <IManyToOneMapper> ( ) ,
				new BidirectionalForeignKeyAssociationManyToOneApplier ( domainInspector ) ,
				new UnidirectionalOneToOneUniqueCascadeApplier ( domainInspector ) ,
				new PolymorphismManyToOneClassApplier ( domainInspector )
			};

			base.manyToOnePath = new List <IPatternApplier <PropertyPath , IManyToOneMapper>> {
				new ManyToOneColumnApplier ( )
			};

			base.oneToMany = new List <IPatternApplier <MemberInfo , IOneToManyMapper>> {
				new PolymorphismOneToManyClassApplier ( domainInspector )
			};

			base.oneToOne = new List <IPatternApplier <MemberInfo , IOneToOneMapper>> {
				new MemberReadOnlyAccessorApplier <IOneToOneMapper> ( ) ,
				new MemberNoSetterToFieldAccessorApplier <IOneToOneMapper> ( ) ,
				new MemberToFieldAccessorApplier <IOneToOneMapper> ( ) ,
				new BidirectionalForeignKeyAssociationOneToOneApplier ( domainInspector ) ,
				new BidirectionalPrimaryKeyAssociationMasterOneToOneApplier ( domainInspector ) ,
				new BidirectionalPrimaryKeyAssociationSlaveOneToOneApplier ( domainInspector )
			};

			base.oneToOnePath = new List <IPatternApplier <PropertyPath , IOneToOneMapper>> ( );

			base.any = new List <IPatternApplier <MemberInfo , IAnyMapper>> {
				new MemberReadOnlyAccessorApplier <IAnyMapper> ( ) ,
				new MemberNoSetterToFieldAccessorApplier <IAnyMapper> ( ) ,
				new MemberToFieldAccessorApplier <IAnyMapper> ( )
			};

			base.anyPath = new List <IPatternApplier <PropertyPath , IAnyMapper>> ( );
			base.manyToMany = new List <IPatternApplier <MemberInfo , IManyToManyMapper>> ( );
			base.manyToManyPath = new List <IPatternApplier <PropertyPath , IManyToManyMapper>> {
				new ManyToManyColumnApplier ( domainInspector )
			};

			base.component = new List <IPatternApplier <Type , IComponentAttributesMapper>> {
				new PolymorphismComponentClassApplier ( domainInspector )
			};

			base.componentParent = new List <IPatternApplier <MemberInfo , IComponentParentMapper>> {
				new MemberNoSetterToFieldAccessorApplier <IComponentParentMapper> ( ) ,
				new MemberToFieldAccessorApplier <IComponentParentMapper> ( )
			};

			base.componentProperty = new List <IPatternApplier <MemberInfo , IComponentAttributesMapper>> {
				new MemberReadOnlyAccessorApplier <IComponentAttributesMapper> ( ) ,
				new MemberNoSetterToFieldAccessorApplier <IComponentAttributesMapper> ( ) ,
				new MemberToFieldAccessorApplier <IComponentAttributesMapper> ( )
			};

			base.elementPath = new List <IPatternApplier <PropertyPath , IElementMapper>> {
				new CollectionOfElementsColumnApplier ( domainInspector )
			};
		}
	}
}