﻿ #region Disclaimer/Info
 
 /////////////////////////////////////////////////////////////////////////////////////////////////
 //
 //   File:		IPostRepository.cs
 //   Website:		http://dexterblogengine.com/
 //   Authors:		http://dexterblogengine.com/About.ashx
 //   Rev:		1
 //   Created:		19/01/2011
 //   Last edit:		19/01/2011
 //   License:		GNU Library General Public License (LGPL)
 //   File:            IPostRepository.cs
 //   For updated news and information please visit http://dexterblogengine.com/
 //   Dexter is hosted to Codeplex at http://dexterblogengine.codeplex.com
 //   For any question contact info@dexterblogengine.com
 //
 ///////////////////////////////////////////////////////////////////////////////////////////////////
 
 #endregion

using System;
using System.Collections.Generic;
using Dexter.Domain.Model;
using Dexter.Domain.Results;

namespace Dexter.Repository {

	/// <summary>
	/// 	Rapresents a repository of <see cref="Post"/>.
	/// </summary>
	public interface IPostRepository : IRepository <Post> {
		///<summary>
		///	Retireve a paged list of approved posts.
		///</summary>
		///<param name = "pageIndex">Index of the page (0 based).</param>
		///<param name = "pageSize">The number of items to retrieve.</param>
		///<returns>An concrete implementation of <see cref="IPagedResult"/>. The result will never be null.</returns>
		/// <exception cref="ArgumentOutOfRangeException">Will be throw if the page size will be lesser than 0.</exception>
		/// <exception cref="ArgumentOutOfRangeException">Will be throw if the page index will be lesser than 1.</exception>
		IPagedResult <Post> GetPublished ( int pageIndex , int pageSize );

		///<summary>
		///	Retireve a paged list of approved posts.
		///</summary>
		///<param name = "pageIndex">Index of the page (0 based).</param>
		///<param name = "pageSize">The number of items to retrieve.</param>
		///<param name="includeFuturesPosts">if set to <c>true</c> the result will include the futures posts.</param>
		///<returns>An concrete implementation of <see cref="IPagedResult{T}"/>. The result will never be null.</returns>
		///<exception cref="ArgumentOutOfRangeException"> If <paramref name="pageIndex"/> is lesser than 0.</exception>
		///<exception cref="ArgumentOutOfRangeException"> If <paramref name="pageSize"/> is lesser than 1.</exception>
		IPagedResult<Post> GetPublished(int pageIndex, int pageSize, bool includeFuturesPosts);

		///<summary>
		///	Retireve a paged list of approved posts.
		///</summary>
		///<param name = "pageIndex">Index of the page (0 based).</param>
		///<param name = "pageSize">The number of items to retrieve.</param>
		///<param name="includeFuturesPosts">if set to <c>true</c> the result will include the futures posts.</param>
		///<param name = "enableTracking">if set to <c>true</c> the tracking system of the return object will be enable.</param>
		///<returns>An concrete implementation of <see cref="IPagedResult{T}"/>. The result will never be null.</returns>
		///<exception cref="ArgumentOutOfRangeException"> If <paramref name="pageIndex"/> is lesser than 0.</exception>
		///<exception cref="ArgumentOutOfRangeException"> If <paramref name="pageSize"/> is lesser than 1.</exception>
		IPagedResult <Post> GetPublished ( int pageIndex , int pageSize , bool includeFuturesPosts, bool enableTracking );

		///<summary>
		///	Retireve a paged list of unapproved posts.
		///</summary>
		///<param name = "pageIndex">Index of the page (0 based).</param>
		///<param name = "pageSize">The number of items to retrieve.</param>
		///<param name="includeFuturesPosts">if set to <c>true</c> the result will include the futures posts.</param>
		///<returns>An concrete implementation of <see cref="IPagedResult{T}"/>. The result will never be null.</returns>
		///<exception cref="ArgumentOutOfRangeException"> If <paramref name="pageIndex"/> is lesser than 0.</exception>
		///<exception cref="ArgumentOutOfRangeException"> If <paramref name="pageSize"/> is lesser than 1.</exception>
		IPagedResult<Post> GetUnpublished(int pageIndex, int pageSize, bool includeFuturesPosts);

		///<summary>
		///	Retireve a paged list of unapproved posts.
		///</summary>
		///<param name = "pageIndex">Index of the page (0 based).</param>
		///<param name = "pageSize">The number of items to retrieve.</param>
		///<param name="includeFuturesPosts">if set to <c>true</c> the result will include the futures posts.</param>
		/// <param name = "enableTracking">if set to <c>true</c> the tracking system of the return object will be enable.</param>
		///<returns>An concrete implementation of <see cref="IPagedResult{T}"/>. The result will never be null.</returns>
		///<exception cref="ArgumentOutOfRangeException"> If <paramref name="pageIndex"/> is lesser than 0.</exception>
		///<exception cref="ArgumentOutOfRangeException"> If <paramref name="pageSize"/> is lesser than 1.</exception>
		IPagedResult <Post> GetUnpublished ( int pageIndex , int pageSize ,bool includeFuturesPosts, bool enableTracking );

		/// <summary>
		/// Retireve a paged list of unapproved posts with <see cref="Item.PublishDate"/> equal to <paramref name="date"/>.
		/// </summary>
		/// <param name="pageIndex">Index of the page (0 based).</param>
		/// <param name="pageSize">The number of items to retrieve.</param>
		/// <param name="date">The date.</param>
		/// <returns>
		/// An concrete implementation of <see cref="IPagedResult{T}"/>. The result will never be null.
		/// </returns>
		/// <exception cref="ArgumentOutOfRangeException"> If <paramref name="pageIndex"/> is lesser than 0.</exception>
		/// <exception cref="ArgumentOutOfRangeException"> If <paramref name="pageSize"/> is lesser than 1.</exception>
		IPagedResult<Post> GetUnpublishedForTheSpecifiedDate ( int pageIndex, int pageSize, DateTime date );

		/// <summary>
		/// Retireve a paged list of unapproved posts with <see cref="Item.PublishDate"/> equal to <paramref name="date"/>.
		/// </summary>
		/// <param name="pageIndex">Index of the page (0 based).</param>
		/// <param name="pageSize">The number of items to retrieve.</param>
		/// <param name="date">The date.</param>
		/// <param name="enableTracking">if set to <c>true</c> the tracking system of the return object will be enable.</param>
		/// <returns>
		/// An concrete implementation of <see cref="IPagedResult{T}"/>. The result will never be null.
		/// </returns>
		/// <exception cref="ArgumentOutOfRangeException"> If <paramref name="pageIndex"/> is lesser than 0.</exception>
		/// <exception cref="ArgumentOutOfRangeException"> If <paramref name="pageSize"/> is lesser than 1.</exception>
		IPagedResult<Post> GetUnpublishedForTheSpecifiedDate ( int pageIndex, int pageSize, DateTime date, bool enableTracking );

		///<summary>
		///	Retireve a paged list of all posts with the property <see cref="Item.CommentEnabled"/> set to <c>false</c>.
		///</summary>
		/// <remarks>
		///		The result will include only published past posts.
		///		Moreover the result hasn't the tracking system enabled
		/// </remarks>
		///<param name = "pageIndex">Index of the page (0 based).</param>
		///<param name = "pageSize">The number of items to retrieve.</param>
		///<returns>An concrete implementation of <see cref="IPagedResult{T}"/>. The result will never be null.</returns>
		///<exception cref="ArgumentOutOfRangeException"> If <paramref name="pageIndex"/> is lesser than 0.</exception>
		///<exception cref="ArgumentOutOfRangeException"> If <paramref name="pageSize"/> is lesser than 1.</exception>
		IPagedResult<Post> GetPublishedPostWithCommentsDisabled ( int pageIndex, int pageSize );

		///<summary>
		///	Retireve a paged list of all posts with the property <see cref="Item.CommentEnabled"/> set to <c>false</c>.
		///</summary>
		/// <remarks>
		///		The result will include only published past posts.
		///		Moreover the result hasn't the tracking system enabled
		/// </remarks>
		///<param name = "pageIndex">Index of the page (0 based).</param>
		///<param name = "pageSize">The number of items to retrieve.</param>
		///<param name = "enableTracking">if set to <c>true</c> the tracking system of the return object will be enable.</param>
		///<returns>An concrete implementation of <see cref="IPagedResult{T}"/>. The result will never be null.</returns>
		///<exception cref="ArgumentOutOfRangeException"> If <paramref name="pageIndex"/> is lesser than 0.</exception>
		///<exception cref="ArgumentOutOfRangeException"> If <paramref name="pageSize"/> is lesser than 1.</exception>
		IPagedResult<Post> GetPublishedPostWithCommentsDisabled ( int pageIndex, int pageSize, bool enableTracking );

		/// <summary>
		/// Retireve a paged list of all posts.
		/// </summary>
		/// <param name="pageIndex">Index of the page (0 based).</param>
		/// <param name="pageSize">The number of items to retrieve.</param>
		/// <param name="startDate">The start date.</param>
		/// <param name="endDate">The end date.</param>
		/// <returns>
		/// An concrete implementation of <see cref="IPagedResult{T}"/>. The result will never be null.
		/// </returns>
		/// <remarks>
		/// The result will include only published.
		/// Moreover the result hasn't the tracking system enabled
		/// </remarks>
		/// <exception cref="ArgumentOutOfRangeException"> If <paramref name="pageIndex"/> is lesser than 0.</exception>
		/// <exception cref="ArgumentOutOfRangeException"> If <paramref name="pageSize"/> is lesser than 1.</exception>
		IPagedResult <Post> GetPublishedPostForSpecifiedPublishDateRange ( int pageIndex , int pageSize , DateTime startDate , DateTime endDate );

		///<summary>
		///	Retireve a paged list of posts.
		///</summary>
		///<param name = "pageIndex">Index of the page (0 based).</param>
		///<param name = "pageSize">The number of items to retrieve.</param>
		///<returns>An concrete implementation of <see cref="IPagedResult{T}"/>. The result will never be null.</returns>
		///<exception cref="ArgumentOutOfRangeException"> If <paramref name="pageIndex"/> is lesser than 0.</exception>
		///<exception cref="ArgumentOutOfRangeException"> If <paramref name="pageSize"/> is lesser than 1.</exception>
		IPagedResult <Post> GetCompleteList ( int pageIndex , int pageSize );

		///<summary>
		///	Retireve a paged list of posts.
		///</summary>
		///<param name = "pageIndex">Index of the page (0 based).</param>
		///<param name = "pageSize">The number of items to retrieve.</param>
		/// <param name = "enableTracking">if set to <c>true</c> the tracking system of the return object will be enable.</param>
		///<returns>An concrete implementation of <see cref="IPagedResult{T}"/>. The result will never be null.</returns>
		///<exception cref="ArgumentOutOfRangeException"> If <paramref name="pageIndex"/> is lesser than 0.</exception>
		///<exception cref="ArgumentOutOfRangeException"> If <paramref name="pageSize"/> is lesser than 1.</exception>
		IPagedResult <Post> GetCompleteList ( int pageIndex , int pageSize , bool enableTracking );

		///<summary>
		///	Return a list of posts for a specified month.
		///</summary>
		///<param name = "month">The id of the month (1 for january, 2 for february, etc).</param>
		///<param name = "year">The year (2010, 1978, etc).</param>
		///<param name = "pageIndex">Index of the page (0 based).</param>
		///<param name = "pageSize">The number of items to retrieve.</param>
		///<returns>An concrete implementation of <see cref="IPagedResult{T}"/>. The result will never be null.</returns>
		///<exception cref="ArgumentOutOfRangeException"> If <paramref name="pageIndex"/> is lesser than 0.</exception>
		///<exception cref="ArgumentOutOfRangeException"> If <paramref name="pageSize"/> is lesser than 1.</exception>
		///<exception cref="ArgumentOutOfRangeException"> If <paramref name="month"/> is not between 1 and 12.</exception>
		///<exception cref="ArgumentOutOfRangeException"> If <paramref name="year"/> is lesser than 1400.</exception>
		IPagedResult <Post> GetByMonth ( int month , int year , int pageIndex , int pageSize );

		///<summary>
		///	Return a list of posts for a specified month.
		///</summary>
		///<param name = "month">The id of the month (1 for january, 2 for february, etc).</param>
		///<param name = "year">The year (2010, 1978, etc).</param>
		///<param name = "pageIndex">Index of the page (0 based).</param>
		///<param name = "pageSize">The number of items to retrieve.</param>
		///<param name="onlyPublished">if set to <c>true</c> the resul will be included only the published posts.</param>
		///<returns>An concrete implementation of <see cref="IPagedResult{T}"/>. The result will never be null.</returns>
		///<exception cref="ArgumentOutOfRangeException"> If <paramref name="pageIndex"/> is lesser than 0.</exception>
		///<exception cref="ArgumentOutOfRangeException"> If <paramref name="pageSize"/> is lesser than 1.</exception>
		///<exception cref="ArgumentOutOfRangeException"> If <paramref name="month"/> is not between 1 and 12.</exception>
		///<exception cref="ArgumentOutOfRangeException"> If <paramref name="year"/> is lesser than 1400.</exception>
		IPagedResult<Post> GetByMonth(int month, int year, int pageIndex, int pageSize, bool onlyPublished);

		///<summary>
		///	Return a list of posts for a specified month.
		///</summary>
		///<param name = "month">The id of the month (1 for january, 2 for february, etc).</param>
		///<param name = "year">The year (2010, 1978, etc).</param>
		///<param name = "pageIndex">Index of the page (0 based).</param>
		///<param name = "pageSize">The number of items to retrieve.</param>
		///<param name="onlyPublished">if set to <c>true</c> the resul will be included only the published posts.</param>
		///<param name = "enableTracking">if set to <c>true</c> the tracking system of the return object will be enable.</param>
		///<returns>An concrete implementation of <see cref="IPagedResult{T}"/>. The result will never be null.</returns>
		///<exception cref="ArgumentOutOfRangeException"> If <paramref name="pageIndex"/> is lesser than 0.</exception>
		///<exception cref="ArgumentOutOfRangeException"> If <paramref name="pageSize"/> is lesser than 1.</exception>
		///<exception cref="ArgumentOutOfRangeException"> If <paramref name="month"/> is not between 1 and 12.</exception>
		///<exception cref="ArgumentOutOfRangeException"> If <paramref name="year"/> is lesser than 1400.</exception>
		IPagedResult <Post> GetByMonth ( int month , int year , int pageIndex , int pageSize , bool onlyPublished, bool enableTracking );

		///<summary>
		///	Return a list approved posts, sorted by publish date.
		///</summary>
		///<param name = "numberOfItems">The number of items.</param>
		///<param name = "includeFutures">if set to <c>true</c> the resulset includes the futures posts.</param>
		///<returns>
		///	Return a list of <see cref="Post"/>.
		///	The return item could be null if there isn't items into the repository.
		///</returns>
		///<exception cref="ArgumentOutOfRangeException"> If <paramref name="numberOfItems"/> is lesser than 1.</exception>
		IEnumerable <Post> GetRecent ( int numberOfItems , bool includeFutures );

		///<summary>
		///	Return a list approved posts, sorted by publish date.
		///</summary>
		///<param name = "numberOfItems">The number of items.</param>
		///<param name = "includeFutures">if set to <c>true</c> the resulset includes the futures posts.</param>
		/// <param name = "enableTracking">if set to <c>true</c> the tracking system of the return object will be enable.</param>
		///<returns>
		///	Return a list of <see cref="Post"/>.
		///	The return item could be null if there isn't items into the repository.
		///</returns>
		///<exception cref="ArgumentOutOfRangeException"> If <paramref name="numberOfItems"/> is lesser than 1.</exception>
		IEnumerable <Post> GetRecent ( int numberOfItems , bool includeFutures , bool enableTracking );

		///<summary>
		///	Return a list most comment posts, sorted by publish date.
		///</summary>
		///<param name = "pageIndex">Index of the page (0 based).</param>
		///<param name = "pageSize">The number of items to retrieve.</param>
		///<returns>An concrete implementation of <see cref="IPagedResult{T}"/>. The result will never be null.</returns>
		///<exception cref="ArgumentOutOfRangeException"> If <paramref name="pageIndex"/> is lesser than 0.</exception>
		///<exception cref="ArgumentOutOfRangeException"> If <paramref name="pageSize"/> is lesser than 1.</exception>
		IPagedResult <Post> GetMostComment ( int pageIndex , int pageSize );

		///<summary>
		///	Return a list most comment posts, sorted by publish date.
		///</summary>
		///<param name = "pageIndex">Index of the page (0 based).</param>
		///<param name = "pageSize">The number of items to retrieve.</param>
		/// <param name = "enableTracking">if set to <c>true</c> the tracking system of the return object will be enable.</param>
		///<returns>An concrete implementation of <see cref="IPagedResult{T}"/>. The result will never be null.</returns>
		///<exception cref="ArgumentOutOfRangeException"> If <paramref name="pageIndex"/> is lesser than 0.</exception>
		///<exception cref="ArgumentOutOfRangeException"> If <paramref name="pageSize"/> is lesser than 1.</exception>
		IPagedResult <Post> GetMostComment ( int pageIndex , int pageSize , bool enableTracking );

		///<summary>
		///	Return a list of posts for a specified category, sorted by publish date.
		///</summary>
		///<param name = "categoryId">The category Id.</param>
		///<param name = "pageIndex">Index of the page (0 based).</param>
		///<param name = "pageSize">The number of items to retrieve.</param>
		///<returns>An concrete implementation of <see cref="IPagedResult{T}"/>. The result will never be null.</returns>
		///<exception cref="ArgumentOutOfRangeException"> If <paramref name="pageIndex"/> is lesser than 0.</exception>
		///<exception cref="ArgumentOutOfRangeException"> If <paramref name="pageSize"/> is lesser than 1.</exception>
		IPagedResult <Post> GetByCategory ( int categoryId , int pageIndex , int pageSize );

		///<summary>
		///	Return a list of posts for a specified category, sorted by publish date.
		///</summary>
		///<param name = "categoryId">The category Id.</param>
		///<param name = "pageIndex">Index of the page (0 based).</param>
		///<param name = "pageSize">The number of items to retrieve.</param>
		/// <param name = "enableTracking">if set to <c>true</c> the tracking system of the return object will be enable.</param>
		///<returns>An concrete implementation of <see cref="IPagedResult{T}"/>. The result will never be null.</returns>
		///<exception cref="ArgumentOutOfRangeException"> If <paramref name="pageIndex"/> is lesser than 0.</exception>
		///<exception cref="ArgumentOutOfRangeException"> If <paramref name="pageSize"/> is lesser than 1.</exception>
		IPagedResult <Post> GetByCategory ( int categoryId , int pageIndex , int pageSize , bool enableTracking );

		///<summary>
		///	Gets the by author.
		///</summary>
		///<param name = "author">The author.</param>
		///<param name = "pageIndex">Index of the page (0 based).</param>
		///<param name = "pageSize">The number of items to retrieve.</param>
		///<returns>An concrete implementation of <see cref="IPagedResult{T}"/>. The result will never be null.</returns>
		///<exception cref="ArgumentOutOfRangeException"> If <paramref name="author"/> is <c>null</c> or <c>empty</c>.</exception>
		///<exception cref="ArgumentOutOfRangeException"> If <paramref name="pageIndex"/> is lesser than 0.</exception>
		///<exception cref="ArgumentOutOfRangeException"> If <paramref name="pageSize"/> is lesser than 1.</exception>
		IPagedResult <Post> GetByAuthor ( string author , int pageIndex , int pageSize );

		///<summary>
		///	Gets the by author.
		///</summary>
		///<param name = "author">The author.</param>
		///<param name = "pageIndex">Index of the page (0 based).</param>
		///<param name = "pageSize">The number of items to retrieve.</param>
		/// <param name = "enableTracking">if set to <c>true</c> the tracking system of the return object will be enable.</param>
		///<returns>An concrete implementation of <see cref="IPagedResult{T}"/>. The result will never be null.</returns>
		///<exception cref="ArgumentOutOfRangeException"> If <paramref name="author"/> is <c>null</c> or <c>empty</c>.</exception>
		///<exception cref="ArgumentOutOfRangeException"> If <paramref name="pageIndex"/> is lesser than 0.</exception>
		///<exception cref="ArgumentOutOfRangeException"> If <paramref name="pageSize"/> is lesser than 1.</exception>
		IPagedResult <Post> GetByAuthor ( string author , int pageIndex , int pageSize , bool enableTracking );

		///<summary>
		///	Retrieve a post for a specified slug.
		///</summary>
		///<param name = "slug">The slug.</param>
		///<returns>
		///	An instance of <see cref="Post"/>.
		///	It could be null if there isn't an item with the specified slug into the repository.
		///</returns>
		///<exception cref="ArgumentOutOfRangeException"> If <paramref name="slug"/> is <c>null</c> or <c>empty</c>.</exception>
		Post GetBySlug ( string slug );

		///<summary>
		///	Retrieve a post for a specified slug.
		///</summary>
		///<param name = "slug">The slug.</param>
		/// <param name = "enableTracking">if set to <c>true</c> the tracking system of the return object will be enable.</param>
		///<returns>
		///	An instance of <see cref="Post"/>.
		///	It could be null if there isn't an item with the specified slug into the repository.
		///</returns>
		///<exception cref="ArgumentOutOfRangeException"> If <paramref name="slug"/> is <c>null</c> or <c>empty</c>.</exception>
		Post GetBySlug ( string slug , bool enableTracking );

		///<summary>
		///	Retrieve a post for a specified key.
		///</summary>
		///<param name = "key">The key of the post.</param>
		///<returns>
		///	An instance of <see cref="Post"/>.
		///	It could be null if there isn't an item with the specified key into the repository.
		///</returns>
		///<exception cref="ArgumentOutOfRangeException"> If <paramref name="key"/> is lesser than 1.</exception>
		Post GetByKey ( int key );

		///<summary>
		///	Retrieve a post for a specified key.
		///</summary>
		///<param name = "key">The key of the post.</param>
		/// <param name = "enableTracking">if set to <c>true</c> the tracking system of the return object will be enable.</param>
		///<returns>
		///	An instance of <see cref="Post"/>.
		///	It could be null if there isn't an item with the specified key into the repository.
		///</returns>
		///<exception cref="ArgumentOutOfRangeException"> If <paramref name="key"/> is lesser than 1.</exception>
		Post GetByKey ( int key , bool enableTracking );

		/// <summary>
		/// 	Deletes the post with the specified slug, from the repository.
		/// </summary>
		/// <param name = "item">The post.</param>
		/// <exception cref="ArgumentNullException">If <paramref name="item"/> is <c>null</c>.</exception>
		void Delete ( Post item );

		/// <summary>
		/// 	Saves the specified post into the repository.
		/// </summary>
		/// <param name = "item">The item.</param>
		/// <exception cref="ArgumentNullException">If <paramref name="item"/> is <c>null</c>.</exception>
		void Save ( Post item );

		/// <summary>
		/// 	Updates the specified post into the repository.
		/// </summary>
		/// <param name = "item">The item.</param>
		/// <exception cref="ArgumentNullException">If <paramref name="item"/> is <c>null</c>.</exception>
		void Update(Post item);

		///<summary>
		///	Retrieve the list of months for all published posts.
		///</summary>
		///<returns>
		///	Return a list of <c>MonthDTO</c> based on <c>PublishDate</c> property of <c>Item</c>.
		///	It could be null if there aren't items into the repository.
		///</returns>
		IList<Month> GetMonths();

		///<summary>
		///	Retrieve the list of months for all posts.
		///</summary>
		/// <param name = "includeFutures">if set to <c>true</c> the result will include the future posts; otherwise, <c>false</c>.</param>
		///<returns>
		///	Return a list of <c>MonthDTO</c> based on <c>PublishDate</c> property of <c>Item</c>.
		///	It could be null if there aren't items into the repository.
		///</returns>
		IList<Month> GetMonths(bool includeFutures);

		/// <summary>
		/// 	Retrieve the list of months for all published posts, filter by an author.
		/// </summary>
		/// <param name = "username">The username.</param>
		/// <returns>
		/// 	Return a list of <c>MonthDTO</c> based on <c>PublishDate</c> property of <c>Item</c>.
		/// 	It could be null if there aren't items into the repository.
		/// </returns>
		///<exception cref="ArgumentOutOfRangeException"> If <paramref name="username"/> is <c>null</c> or <c>empty</c>.</exception>
		IList<Month> GetMonths(string username);

		/// <summary>
		/// 	Retrieve the list of months for all posts, filter by an author.
		/// </summary>
		/// <param name = "username">The username.</param>
		/// <param name = "includeFutures">if set to <c>true</c> the result will include the future posts; otherwise, <c>false</c>.</param>
		/// <returns>
		/// 	Return a list of <c>MonthDTO</c> based on <c>PublishDate</c> property of <c>Item</c>.
		/// 	It could be null if there aren't items into the repository.
		/// </returns>
		///<exception cref="ArgumentOutOfRangeException"> If <paramref name="username"/> is <c>null</c> or <c>empty</c>.</exception>
		IList<Month> GetMonths(string username, bool includeFutures);
	}
}
