﻿ #region Disclaimer/Info
 
 /////////////////////////////////////////////////////////////////////////////////////////////////
 //
 //   File:		ITrackBackRepository.cs
 //   Website:		http://dexterblogengine.com/
 //   Authors:		http://dexterblogengine.com/About.ashx
 //   Rev:		1
 //   Created:		19/01/2011
 //   Last edit:		19/01/2011
 //   License:		GNU Library General Public License (LGPL)
 //   File:            ITrackBackRepository.cs
 //   For updated news and information please visit http://dexterblogengine.com/
 //   Dexter is hosted to Codeplex at http://dexterblogengine.codeplex.com
 //   For any question contact info@dexterblogengine.com
 //
 ///////////////////////////////////////////////////////////////////////////////////////////////////
 
 #endregion

using System;
using Dexter.Domain.Model;
using Dexter.Domain.Results;

namespace Dexter.Repository {

	/// <summary>
	/// 	Rapresents a repository of <see cref="TrackBack"/>.
	/// </summary>
	public interface ITrackBackRepository : IRepository <TrackBack> {
		///<summary>
		///	Adds the specified <c>TrackBack</c> to the repository.
		///</summary>
		///<param name = "item">The item to save.</param>
		///<exception cref="ArgumentNullException"> If <paramref name="item"/> is <c>null</c>.</exception>
		void Save ( TrackBack item );

		/// <summary>
		/// 	Determines whether is first ping back for the specified post slug.
		/// </summary>
		/// <param name = "postSlug">The post slug.</param>
		/// <param name = "url">The URL.</param>
		/// <returns>
		/// 	<c>true</c> if is first ping back; otherwise, <c>false</c>.
		/// </returns>
		///<exception cref="ArgumentNullException"> If <paramref name="postSlug"/> is <c>null</c> or <c>empty</c>.</exception>
		///<exception cref="ArgumentNullException"> If <paramref name="url"/> is <c>null</c>.</exception>
		bool IsFirstPingBack ( string postSlug , Uri url );

		/// <summary>
		/// 	Determines whether is first ping back for the specified post slug.
		/// </summary>
		/// <param name = "postKey">The post key.</param>
		/// <param name = "url">The URL.</param>
		/// <returns>
		/// 	<c>true</c> if is first ping back; otherwise, <c>false</c>.
		/// </returns>
		///<exception cref="ArgumentNullException"> If <paramref name="postKey"/> is lesser than 1.</exception>
		///<exception cref="ArgumentNullException"> If <paramref name="url"/> is <c>null</c>.</exception>
		bool IsFirstPingBack ( int postKey , Uri url );

		///<summary>
		///	Gets the trackbacks.
		///</summary>
		///<param name = "pageIndex">Index of the page (0 based).</param>
		///<param name = "pageSize">The number of items to retrieve.</param>
		///<returns>An concrete implementation of <see cref="IPagedResult"/>. The result will never be null.</returns>
		///<exception cref="ArgumentOutOfRangeException"> If <paramref name="pageIndex"/> is lesser than 0.</exception>
		///<exception cref="ArgumentOutOfRangeException"> If <paramref name="pageSize"/> is lesser than 1.</exception>
		IPagedResult <TrackBack> GetTrackBack ( int pageSize , int pageIndex );

		///<summary>
		///	Gets the trackbacks.
		///</summary>
		///<param name = "pageIndex">Index of the page (0 based).</param>
		///<param name = "pageSize">The number of items to retrieve.</param>
		///<param name = "enableTracking">if set to <c>true</c> the tracking system of the return object will be enable.</param>
		///<returns>An concrete implementation of <see cref="IPagedResult{T}"/>. The result will never be null.</returns>
		///<exception cref="ArgumentOutOfRangeException"> If <paramref name="pageIndex"/> is lesser than 0.</exception>
		///<exception cref="ArgumentOutOfRangeException"> If <paramref name="pageSize"/> is lesser than 1.</exception>
		IPagedResult <TrackBack> GetTrackBack ( int pageSize , int pageIndex , bool enableTracking );

		///<summary>
		///	Gets the trackbacks for a specific post.
		///</summary>
		///<param name = "itemId">The post Id.</param>
		///<param name = "pageIndex">Index of the page (0 based).</param>
		///<param name = "pageSize">The number of items to retrieve.</param>
		///<returns>An concrete implementation of <see cref="IPagedResult{T}"/>. The result will never be null.</returns>
		///<exception cref="ArgumentOutOfRangeException"> If <paramref name="pageIndex"/> is lesser than 0.</exception>
		///<exception cref="ArgumentOutOfRangeException"> If <paramref name="pageSize"/> is lesser than 1.</exception>
		///<exception cref="ArgumentOutOfRangeException"> If <paramref name="itemId"/> is lesser than 1.</exception>
		IPagedResult <TrackBack> GetTrackBack ( int itemId , int pageSize , int pageIndex );

		///<summary>
		///	Gets the trackbacks for a specific post.
		///</summary>
		///<param name = "itemId">The post Id.</param>
		///<param name = "pageIndex">Index of the page (0 based).</param>
		///<param name = "pageSize">The number of items to retrieve.</param>
		///<param name = "enableTracking">if set to <c>true</c> the tracking system of the return object will be enable.</param>
		///<returns>An concrete implementation of <see cref="IPagedResult{T}"/>. The result will never be null.</returns>
		///<exception cref="ArgumentOutOfRangeException"> If <paramref name="pageIndex"/> is lesser than 0.</exception>
		///<exception cref="ArgumentOutOfRangeException"> If <paramref name="pageSize"/> is lesser than 1.</exception>
		///<exception cref="ArgumentOutOfRangeException"> If <paramref name="itemId"/> is lesser than 1.</exception>
		IPagedResult <TrackBack> GetTrackBack ( int itemId , int pageSize , int pageIndex , bool enableTracking );

		///<summary>
		///	Gets the trackbacks for a specific post.
		///</summary>
		///<param name = "postSlug">The post slug.</param>
		///<param name = "pageIndex">Index of the page (0 based).</param>
		///<param name = "pageSize">The number of items to retrieve.</param>
		///<returns>An concrete implementation of <see cref="IPagedResult{T}"/>. The result will never be null.</returns>
		///<exception cref="ArgumentOutOfRangeException"> If <paramref name="postSlug"/> is <c>null</c> or <c>empty</c>.</exception>
		///<exception cref="ArgumentOutOfRangeException"> If <paramref name="pageIndex"/> is lesser than 0.</exception>
		///<exception cref="ArgumentOutOfRangeException"> If <paramref name="pageSize"/> is lesser than 1.</exception>
		IPagedResult <TrackBack> GetTrackBack ( string postSlug , int pageSize , int pageIndex );

		///<summary>
		///	Gets the trackbacks for a specific post.
		///</summary>
		///<param name = "postSlug">The post slug.</param>
		///<param name = "pageIndex">Index of the page (0 based).</param>
		///<param name = "pageSize">The number of items to retrieve.</param>
		///<param name = "enableTracking">if set to <c>true</c> the tracking system of the return object will be enable.</param>
		///<returns>An concrete implementation of <see cref="IPagedResult{T}"/>. The result will never be null.</returns>
		///<exception cref="ArgumentOutOfRangeException"> If <paramref name="postSlug"/> is <c>null</c> or <c>empty</c>.</exception>
		///<exception cref="ArgumentOutOfRangeException"> If <paramref name="pageIndex"/> is lesser than 0.</exception>
		///<exception cref="ArgumentOutOfRangeException"> If <paramref name="pageSize"/> is lesser than 1.</exception>
		IPagedResult <TrackBack> GetTrackBack ( string postSlug , int pageSize , int pageIndex , bool enableTracking );
	}
}
