﻿ #region Disclaimer/Info
 
 /////////////////////////////////////////////////////////////////////////////////////////////////
 //
 //   File:		WebViewPage (generic).cs
 //   Website:		http://dexterblogengine.com/
 //   Authors:		http://dexterblogengine.com/About.ashx
 //   Rev:		1
 //   Created:		19/01/2011
 //   Last edit:		19/01/2011
 //   License:		GNU Library General Public License (LGPL)
 // 
 //   For updated news and information please visit http://dexterblogengine.com/
 //   Dexter is hosted to Codeplex at http://dexterblogengine.codeplex.com
 //   For any question contact info@dexterblogengine.com
 //
 ///////////////////////////////////////////////////////////////////////////////////////////////////
 
 #endregion

using Dexter.Dependency;
using Dexter.Domain.Model;
using Dexter.Services;
using Dexter.Web.Mvc.ResourceCombiner;
using Dexter.Web.Utils;

namespace Dexter.Web.Mvc.ViewPages {
	/// <summary>
	/// 	Represents the properties and methods that are needed in order to render a view that uses ASP.NET Razor syntax.
	/// </summary>
	/// <typeparam name = "TModel">The type of the model.</typeparam>
	public abstract class WebViewPage <TModel> : System.Web.Mvc.WebViewPage <TModel> , IDexterView {
		ICombiner combiner;
		SiteConfiguration siteConfiguration;
		IUrlBuilder urlBuilder;
		IWebUtils utils;

		#region IDexterView Members

		/// <summary>
		/// 	Return the instance of <see cref = "IUrlBuilder" />.
		/// </summary>
		/// <value>The U.</value>
		public IUrlBuilder U {
			get { return urlBuilder; }
		}

		/// <summary>
		/// 	Return the instance of <see cref = "SiteConfiguration" />.
		/// </summary>
		/// <value>The conf.</value>
		public SiteConfiguration C {
			get { return siteConfiguration; }
		}

		/// <summary>
		/// 	Gets the combiner.
		/// </summary>
		/// <value>The combiner.</value>
		public ICombiner Combiner {
			get { return combiner; }
		}

		/// <summary>
		/// Gets the utils.
		/// </summary>
		/// <value>The utils.</value>
		public IWebUtils Utils {
			get { return utils; }
		}

		/// <summary>
		/// Gets a value indicating whether this instance is moderator.
		/// </summary>
		/// <value>
		/// 	<c>true</c> if this instance is moderator; otherwise, <c>false</c>.
		/// </value>
		public bool IsModerator {
			get { return base.Context.User.IsInRole ( "Moderator" ); }
		}

		/// <summary>
		/// 	Check if the specified username is the current logged user.
		/// </summary>
		/// <param name = "username">The username.</param>
		/// <returns></returns>
		public bool CurrentuserIsEqualtTo ( string username ) {
			return base.Context.User.Identity.Name == username;
		}

		#endregion

		/// <summary>
		/// 	Inits the helpers.
		/// </summary>
		public override void InitHelpers ( ) {
			base.InitHelpers ( );

			combiner = DexterContainer.Resolve <ICombiner> ( );
			utils = DexterContainer.Resolve<IWebUtils> ( );
			urlBuilder = DexterContainer.Resolve<IUrlBuilder> ( );
			siteConfiguration = DexterContainer.Resolve<IConfigurationService> ( ).Configuration;
		}
	}
}
