﻿using System;
using Dexter.Caching.AppFabric;
using Dexter.Caching.AppFabric.Test.Helpers;
using NUnit.Framework;
using SharpTestsEx;

namespace Gaia.Alp.Caching.AppFabric.IntegrationTest {
	[TestFixture]
	public class AppFabricCacheProviderTest {

		#region Add Method Tests

		[Test]
		public void AddMethodWithThreeParameters_WithNullKey_ShoudlThrowNewArgumentNullException ( ) {
			var sut = new AppFabricCacheProvider ( CacheProviderHelper.GetFactoryStub (  ) );

			Executing.This ( ( ) => sut.Add ( null , null , TimeSpan.Zero ) )
				.Should ( )
				.Throw <ArgumentNullException> ( )
				.And
				.Exception
				.ParamName
				.Should ( )
				.Be
				.EqualTo ( "key" );
		}

		[Test]
		public void AddMethodWithThreeParameters_WithEmptyKey_ShoudlThrowNewArgumentException ( ) {
			var sut = new AppFabricCacheProvider ( CacheProviderHelper.GetFactoryStub ( ) );

			Executing.This ( ( ) => sut.Add ( "" , null , TimeSpan.Zero ) )
				.Should ( )
				.Throw <ArgumentException> ( )
				.And
				.Exception
				.ParamName
				.Should ( )
				.Be
				.EqualTo ( "key" );
		}

		[Test]
		public void AddMethodWithFourParameters_WithNullKey_ShoudlThrowNewArgumentNullException ( ) {
			var sut = new AppFabricCacheProvider ( CacheProviderHelper.GetFactoryStub ( ) );

			Executing.This ( ( ) => sut.Add ( null, null, TimeSpan.Zero,null ) )
				.Should ( )
				.Throw<ArgumentNullException> ( )
				.And
				.Exception
				.ParamName
				.Should ( )
				.Be
				.EqualTo ( "key" );
		}

		[Test]
		public void AddMethodWithFourParameters_WithEmptyKey_ShoudlThrowNewArgumentException ( ) {
			var sut = new AppFabricCacheProvider ( CacheProviderHelper.GetFactoryStub ( ) );

			Executing.This ( ( ) => sut.Add ( "", null, TimeSpan.Zero ,null) )
				.Should ( )
				.Throw<ArgumentException> ( )
				.And
				.Exception
				.ParamName
				.Should ( )
				.Be
				.EqualTo ( "key" );
		}

		[Test]
		public void AddMethodWithFiveParameters_WithNullKey_ShoudlThrowNewArgumentNullException ( ) {
			var sut = new AppFabricCacheProvider ( CacheProviderHelper.GetFactoryStub ( ) );

			Executing.This ( ( ) => sut.Add ( null, null, TimeSpan.Zero, null,null ) )
				.Should ( )
				.Throw<ArgumentNullException> ( )
				.And
				.Exception
				.ParamName
				.Should ( )
				.Be
				.EqualTo ( "key" );
		}

		[Test]
		public void AddMethodWithFiveParameters_WithEmptyKey_ShoudlThrowNewArgumentException ( ) {
			var sut = new AppFabricCacheProvider ( CacheProviderHelper.GetFactoryStub ( ) );

			Executing.This ( ( ) => sut.Add ( "", null, TimeSpan.Zero, null, null ) )
				.Should ( )
				.Throw<ArgumentException> ( )
				.And
				.Exception
				.ParamName
				.Should ( )
				.Be
				.EqualTo ( "key" );
		}

		[Test]
		public void AddMethodWithFiveParameters_WithValidKeyAndNullRegion_ShoudlThrowNewArgumentNullException ( ) {
			var sut = new AppFabricCacheProvider ( CacheProviderHelper.GetFactoryStub ( ) );

			Executing.This ( ( ) => sut.Add ( "key", null, TimeSpan.Zero, null, null ) )
				.Should ( )
				.Throw<ArgumentNullException> ( )
				.And
				.Exception
				.ParamName
				.Should ( )
				.Be
				.EqualTo ( "region" );
		}

		[Test]
		public void AddMethodWithFiveParameters_WithValidKeyAndEmptyRegion_ShoudlThrowNewArgumentException ( ) {
			var sut = new AppFabricCacheProvider ( CacheProviderHelper.GetFactoryStub ( ) );

			Executing.This ( ( ) => sut.Add ( "key", null, TimeSpan.Zero, "", null ) )
				.Should ( )
				.Throw<ArgumentException> ( )
				.And
				.Exception
				.ParamName
				.Should ( )
				.Be
				.EqualTo ( "region" );
		}

		#endregion

		#region AddOrReplace Method Tests

		[Test]
		public void AddOrReplaceMethodWithThreeParameters_WithNullKey_ShoudlThrowNewArgumentNullException ( ) {
			var sut = new AppFabricCacheProvider ( CacheProviderHelper.GetFactoryStub ( ) );

			Executing.This ( ( ) => sut.Put ( null, null, TimeSpan.Zero ) )
				.Should ( )
				.Throw<ArgumentNullException> ( )
				.And
				.Exception
				.ParamName
				.Should ( )
				.Be
				.EqualTo ( "key" );
		}

		[Test]
		public void AddOrReplaceMethodWithThreeParameters_WithEmptyKey_ShoudlThrowNewArgumentException ( ) {
			var sut = new AppFabricCacheProvider ( CacheProviderHelper.GetFactoryStub ( ) );

			Executing.This ( ( ) => sut.Put ( "", null, TimeSpan.Zero ) )
				.Should ( )
				.Throw<ArgumentException> ( )
				.And
				.Exception
				.ParamName
				.Should ( )
				.Be
				.EqualTo ( "key" );
		}

		[Test]
		public void AddOrReplaceMethodWithFourParameters_WithNullKey_ShoudlThrowNewArgumentNullException ( ) {
			var sut = new AppFabricCacheProvider ( CacheProviderHelper.GetFactoryStub ( ) );

			Executing.This ( ( ) => sut.Put ( null, null, TimeSpan.Zero, null ) )
				.Should ( )
				.Throw<ArgumentNullException> ( )
				.And
				.Exception
				.ParamName
				.Should ( )
				.Be
				.EqualTo ( "key" );
		}

		[Test]
		public void AddOrReplaceMethodWithFourParameters_WithEmptyKey_ShoudlThrowNewArgumentException ( ) {
			var sut = new AppFabricCacheProvider ( CacheProviderHelper.GetFactoryStub ( ) );

			Executing.This ( ( ) => sut.Put ( "", null, TimeSpan.Zero, null ) )
				.Should ( )
				.Throw<ArgumentException> ( )
				.And
				.Exception
				.ParamName
				.Should ( )
				.Be
				.EqualTo ( "key" );
		}

		[Test]
		public void AddOrReplaceMethodWithFiveParameters_WithNullKey_ShoudlThrowNewArgumentNullException ( ) {
			var sut = new AppFabricCacheProvider ( CacheProviderHelper.GetFactoryStub ( ) );

			Executing.This ( ( ) => sut.Put ( null, null, TimeSpan.Zero, null, null ) )
				.Should ( )
				.Throw<ArgumentNullException> ( )
				.And
				.Exception
				.ParamName
				.Should ( )
				.Be
				.EqualTo ( "key" );
		}

		[Test]
		public void AddOrReplaceMethodWithFiveParameters_WithEmptyKey_ShoudlThrowNewArgumentException ( ) {
			var sut = new AppFabricCacheProvider ( CacheProviderHelper.GetFactoryStub ( ) );

			Executing.This ( ( ) => sut.Put ( "", null, TimeSpan.Zero, null, null ) )
				.Should ( )
				.Throw<ArgumentException> ( )
				.And
				.Exception
				.ParamName
				.Should ( )
				.Be
				.EqualTo ( "key" );
		}

		[Test]
		public void AddOrReplaceMethodWithFiveParameters_WithValidKeyAndNullRegion_ShoudlThrowNewArgumentNullException ( ) {
			var sut = new AppFabricCacheProvider ( CacheProviderHelper.GetFactoryStub ( ) );

			Executing.This ( ( ) => sut.Put ( "key", null, TimeSpan.Zero, null, null ) )
				.Should ( )
				.Throw<ArgumentNullException> ( )
				.And
				.Exception
				.ParamName
				.Should ( )
				.Be
				.EqualTo ( "region" );
		}

		[Test]
		public void AddOrReplaceMethodWithFiveParameters_WithValidKeyAndEmptyRegion_ShoudlThrowNewArgumentException ( ) {
			var sut = new AppFabricCacheProvider ( CacheProviderHelper.GetFactoryStub ( ) );

			Executing.This ( ( ) => sut.Put ( "key", null, TimeSpan.Zero, "", null ) )
				.Should ( )
				.Throw<ArgumentException> ( )
				.And
				.Exception
				.ParamName
				.Should ( )
				.Be
				.EqualTo ( "region" );
		}

		#endregion

		#region BulkGet Method Tests

		[Test]
		public void BulkGet_WithNullKeys_ShoudlThrowNewArgumentNullException() {
			var sut = new AppFabricCacheProvider ( CacheProviderHelper.GetFactoryStub ( ) );

			Executing.This ( ( ) => sut.BulkGet ( null, "region1") )
				.Should ( )
				.Throw<ArgumentNullException> ( )
				.And
				.Exception
				.ParamName
				.Should ( )
				.Be
				.EqualTo ( "keys" );
		}

		[Test]
		public void BulkGet_WithNullKey_ShoudlThrowNewArgumentNullException ( ) {
			var sut = new AppFabricCacheProvider ( CacheProviderHelper.GetFactoryStub ( ) );

			Executing.This ( ( ) => sut.BulkGet ( new []{"key"}, null ) )
				.Should ( )
				.Throw<ArgumentNullException> ( )
				.And
				.Exception
				.ParamName
				.Should ( )
				.Be
				.EqualTo ( "region" );
		}

		[Test]
		public void BulkGet_WithEmptyKey_ShoudlThrowNewArgumentException ( ) {
			var sut = new AppFabricCacheProvider ( CacheProviderHelper.GetFactoryStub ( ) );

			Executing.This ( ( ) => sut.BulkGet ( new[] { "key" }, "" ) )
				.Should ( )
				.Throw<ArgumentException> ( )
				.And
				.Exception
				.ParamName
				.Should ( )
				.Be
				.EqualTo ( "region" );
		}

		#endregion
	}
}