﻿using System;
using System.Collections.Generic;
using log4net.Appender;
using log4net.Core;

namespace Dexter.Dependency.Autofac.Test.Logging.Helpers {
	public class TestAppender : AppenderSkeleton {
		#region Public Instance Constructors

		/// <summary>
		/// 	This is the list of logs
		/// </summary>
		[ThreadStatic] public static List <LoggingEvent> Logs;

		#endregion Public Instance Constructors

		#region Override implementation of AppenderSkeleton

		/// <summary>
		/// 	This appender requires a <see cref = "log4net.Layout" /> to be set.
		/// </summary>
		/// <value><c>true</c></value>
		/// <remarks>
		/// 	<para>
		/// 		This appender requires a <see cref = "log4net.Layout" /> to be set.
		/// 	</para>
		/// </remarks>
		protected override bool RequiresLayout {
			get { return true; }
		}

		/// <summary>
		/// 	This method is called by the <see cref = "AppenderSkeleton.DoAppend(LoggingEvent)" /> method.
		/// 	It calls static method <see cref = "AppenderService.Append" /> that is used to notify the log
		/// 	to all registered clients.
		/// </summary>
		/// <param name = "loggingEvent">The event to log.</param>
		/// <remarks>
		/// 	<para>
		/// 		Send the event to all registered listener.
		/// 	</para>
		/// 	<para>
		/// 		Exceptions are passed to the <see cref = "AppenderSkeleton.ErrorHandler" />.
		/// 	</para>
		/// </remarks>
		protected override void Append ( LoggingEvent loggingEvent ) {
			if ( Logs == null ) {
				Logs = new List <LoggingEvent> ( );
			}

			Logs.Add ( loggingEvent );
		}

		/// <summary>
		/// 	Close the appender
		/// </summary>
		protected override void OnClose ( ) {}

		#endregion Override implementation of AppenderSkeleton
	}
}