﻿using Autofac;
using Autofac.Builder;
using Dexter.Dependency.Autofac.Module;
using Dexter.Dependency.Autofac.Test.Logging.Helpers;
using Dexter.Logger.Contracts;
using Dexter.Logger.Log4Net;
using NUnit.Framework;
using SharpTestsEx;

namespace Dexter.Dependency.Autofac.Test.Logging {

	[TestFixture]
	public class LoggerWrapperTest {
		[Test]
		public void VerifyThatLoggerFacilityResolvesObjectsThatDependsonIlogger ( ) {
			ContainerBuilder builder = new ContainerBuilder (  );
			ConfigureBuilder ( builder );

			builder.RegisterType <DependFromILogger> ( );

			var container = builder.Build ( );

			var instance = container.Resolve<DependFromILogger> ( );

			instance.Should ( ).Not.Be.Null ( );
		}

		[Test]
		public void VerifyLoggerHasTheNameOfDependantObject ( ) {
			ContainerBuilder builder = new ContainerBuilder ( );
			ConfigureBuilder ( builder );

			builder.RegisterType<DependFromILogger> ( );
			var container = builder.Build ( );

			DependFromILogger obj = container.Resolve<DependFromILogger> ( );
			obj.Log ( "this is a message" );
			var logs = TestAppender.Logs;
			logs.Should ( ).Have.Count.EqualTo ( 1 );
			logs[0].LoggerName.Should ( ).Be.EqualTo ( "Dexter.Dependency.Autofac.Test.Logging.Helpers.DependFromILogger" );
			logs[0].MessageObject.Should ( ).Be.EqualTo ( "this is a message" );
			TestAppender.Logs.Clear ( );
		}

		[Test]
		public void VerifyThatLoggerFactoryIsRegisteredCorrectly ( ) {
			ContainerBuilder builder = new ContainerBuilder ( );
			ConfigureBuilder ( builder );

			var container = builder.Build ( );

			ILoggerFactory resolvedLogger = container.Resolve<ILoggerFactory> ( );

			resolvedLogger.Should ( ).Not.Be.Null ( );
		}

		static void ConfigureBuilder ( ContainerBuilder builder ) {
			builder.Register <ILoggerFactory> ( x => new Log4NetFactory ( ) );
			builder.RegisterType <Log4NetLogger> ( ).Named <ILogger> ( "Log4NetLogger" );
			builder.RegisterModule ( new LoggingModule ( ) );
		}
	}
}