﻿using Castle.Core;
using Castle.MicroKernel.Registration;
using Castle.Windsor;
using Dexter.Dependency.Castle.Resolver;
using Dexter.Dependency.Castle.Test.Logging.Helpers;
using Dexter.Logger.Contracts;
using Dexter.Logger.Log4Net;
using NUnit.Framework;
using SharpTestsEx;

namespace Dexter.Dependency.Castle.Test.Logging {

	[TestFixture]
	public class LoggerWrapperTest {
		[Test]
		public void VerifyThatLoggerFacilityResolvesObjectsThatDependsonIlogger ( ) {
			using ( WindsorContainer container = new WindsorContainer ( ) ) {
				ConfigureContainer ( container );

				container.Register ( Component.For <DependFromILogger> ( ) );
				var instance = container.Resolve <DependFromILogger> ( );

				instance.Should ( ).Not.Be.Null ( );
			}
		}

		[Test]
		public void VerifyLoggerHasTheNameOfDependantObject ( ) {
			using ( WindsorContainer container = new WindsorContainer ( ) ) {
				ConfigureContainer ( container );

				container.Register ( Component.For <DependFromILogger> ( ) );
				DependFromILogger obj = container.Resolve <DependFromILogger> ( );
				obj.Log ( "this is a message" );
				var logs = TestAppender.Logs;
				logs.Should ( ).Have.Count.EqualTo ( 1 );
				logs[0].LoggerName.Should ( ).Be.EqualTo ( "Dexter.Dependency.Castle.Test.Logging.Helpers.DependFromILogger" );
				logs[ 0 ].MessageObject.Should ( ).Be.EqualTo ( "this is a message" );
				TestAppender.Logs.Clear ( );
			}
		}

		[Test]
		public void VerifyThatLoggerFactoryIsRegisteredCorrectly ( ) {
			using ( WindsorContainer container = new WindsorContainer ( ) ) {
				ConfigureContainer ( container );

				ILoggerFactory resolvedLogger = container.Resolve <ILoggerFactory> ( );

				resolvedLogger.Should ( ).Not.Be.Null ( );
			}
		}

		static void ConfigureContainer ( WindsorContainer container ) {
			container.Kernel.Resolver.AddSubResolver ( new LoggerSubDependencyResolver ( container.Kernel ) );
			var loggerFactory = Component.For ( typeof ( ILoggerFactory ) )
				.ImplementedBy ( typeof ( Log4NetFactory ) )
				.LifeStyle.Is ( LifestyleType.Singleton );

			var logger = Component.For ( typeof ( ILogger ) )
				.ImplementedBy ( typeof ( Log4NetLogger ) )
				.LifeStyle.Is ( LifestyleType.Singleton );

			container.Register ( logger );
			container.Register ( loggerFactory );
		}
	}
}