﻿#region Disclaimer/Info

/////////////////////////////////////////////////////////////////////////////////////////////////
//
//   File:		BlogRollTest.cs
//   Website:		http://dexterblogengine.com/
//   Authors:		http://dexterblogengine.com/About.ashx
//   Rev:		1
//   Created:		22/11/2010
//   Last edit:		19/01/2011
//   License:		GNU Library General Public License (LGPL)
//   File:            BlogRollTest.cs
//   For updated news and information please visit http://dexterblogengine.com/
//   Dexter is hosted to Codeplex at http://dexterblogengine.codeplex.com
//   For any question contact info@dexterblogengine.com
//
///////////////////////////////////////////////////////////////////////////////////////////////////

#endregion

using System;
using Dexter.Dependency;
using Dexter.Domain.Model;
using Dexter.Logger.Concretes;
using Dexter.Logger.Contracts;
using NUnit.Framework;
using SharpTestsEx;

namespace Dexter.Domain.Test.Model {

	[TestFixture]
	public class BlogRollTest {
		
		[TestFixtureSetUp]
		public void Startup() {
			var stubContainer = new Moq.Mock<IDexterContainer>();
			stubContainer.Setup(x => x.Resolve<ILogger>()).Returns(NullLogger.Instance);
			DexterContainer.SetCurrent ( stubContainer.Object );
		}

		[Test]
		[Category ( "Builder" )]
		public void CreateNewBlogrollShouldThrowArgumentNullExceptionWithNullLinkProperty ( ) {
			Executing.This ( ( ) => {
				{
					BlogRoll.CreateNewBlogRoll ( "tostring.it" , null );
				}
			                 } )
				.Should ( )
				.Throw <ArgumentNullException> ( )
				.And
				.Exception
				.ParamName
				.Should ( )
				.Be
				.EqualTo ( "link" );
		}

		[Test]
		[Category ( "Builder" )]
		public void CreateNewBlogrollShouldThrowArgumentNullExceptionWithNullNameProperty ( ) {
			Executing.This ( ( ) => {
				{
					BlogRoll.CreateNewBlogRoll ( null , new Uri ( "http://www.tostring.it" ) );
				}
			                 } )
				.Should ( )
				.Throw <ArgumentNullException> ( )
				.And
				.Exception
				.ParamName
				.Should ( )
				.Be
				.EqualTo ( "name" );
		}

		[Test]
		[Category ( "Builder" )]
		public void CreateNewBlogrollShouldThrowArgumentOutOfRangeExceptionWithEmptyNameProperty ( ) {
			Executing.This ( ( ) => {
				{
					BlogRoll.CreateNewBlogRoll ( string.Empty , new Uri ( "http://www.tostring.it" ) );
				}
			                 } )
				.Should ( )
				.Throw <ArgumentOutOfRangeException> ( )
				.And
				.Exception
				.ParamName
				.Should ( )
				.Be
				.EqualTo ( "name" );
		}

		[Test]
		[Category ( "Builder" )]
		public void CreateNewBlogrollShouldThrowFormatExceptionWithNullLinkPropertyValue ( ) {
			Executing.This ( ( ) => {
				{
					BlogRoll.CreateNewBlogRoll ( "tostring.it" , null );
				}
			                 } )
				.Should ( )
				.Throw <ArgumentNullException> ( );
		}

		[Test]
		[Category ( "Validation" )]
		public void ValidateForSaveShouldNoThrowExceptionWithZeroPositionValue ( ) {
			Executing.This ( ( ) => {
				{
					BlogRoll item = new BlogRoll ( "tostring.it" , new Uri ( "http://www.tostring.it" ) );
					item.Position = 0;
					item.ValidateForSave ( );
				}
			                 } )
				.Should ( )
				.NotThrow ( );
		}

		[Test]
		[Category ( "Validation" )]
		public void ValidateForSaveShouldThrowArgumentExceptionWithIDPropertyGreatherThenZero ( ) {
			Executing.This ( ( ) => {
			                 	BlogRoll item = new BlogRoll ( "tostring" , new Uri ( "http://www.tostring.it" ) ) {
			                 		Id = 5
			                 	};
			                 	item.ValidateForSave ( );
			                 } )
				.Should ( )
				.Throw <ArgumentException> ( )
				.And
				.Exception
				.ParamName
				.Should ( )
				.Be
				.EqualTo ( "Id" );
		}

		[Test]
		[Category ( "Validation" )]
		public void ValidateForSaveShouldThrowArgumentNullExceptionWithNullNameProperty ( ) {
			Executing.This ( ( ) => {
				{
					BlogRoll item = new BlogRoll ( null , new Uri ( "http://www.tostring.it" ) );
					item.ValidateForSave ( );
				}
			                 } )
				.Should ( )
				.Throw <ArgumentNullException> ( )
				.And
				.Exception
				.ParamName
				.Should ( )
				.Be
				.EqualTo ( "Name" );
		}

		[Test]
		[Category ( "Validation" )]
		public void ValidateForSaveShouldThrowArgumentOutOfRangeExceptionWithEmptyNameProperty ( ) {
			Executing.This ( ( ) => {
				{
					BlogRoll item = new BlogRoll ( string.Empty , new Uri ( "http://www.tostring.it" ) );
					item.ValidateForSave ( );
				}
			                 } )
				.Should ( )
				.Throw <ArgumentOutOfRangeException> ( )
				.And
				.Exception
				.ParamName
				.Should ( )
				.Be
				.EqualTo ( "Name" );
		}

		[Test]
		[Category ( "Validation" )]
		public void ValidateForSaveShouldThrowArgumentOutOfRangeExceptionWithNegativePositionValue ( ) {
			Executing.This ( ( ) => {
				{
					BlogRoll item = new BlogRoll ( "tostring.it" , new Uri ( "http://www.tostring.it" ) );
					item.Position = -1;
					item.ValidateForSave ( );
				}
			                 } )
				.Should ( )
				.Throw <ArgumentOutOfRangeException> ( );
		}
	}
}