﻿#region Disclaimer/Info

/////////////////////////////////////////////////////////////////////////////////////////////////
//
//   File:		CategoryTest.cs
//   Website:		http://dexterblogengine.com/
//   Authors:		http://dexterblogengine.com/About.ashx
//   Rev:		1
//   Created:		22/11/2010
//   Last edit:		19/01/2011
//   License:		GNU Library General Public License (LGPL)
//   File:            CategoryTest.cs
//   For updated news and information please visit http://dexterblogengine.com/
//   Dexter is hosted to Codeplex at http://dexterblogengine.codeplex.com
//   For any question contact info@dexterblogengine.com
//
///////////////////////////////////////////////////////////////////////////////////////////////////

#endregion

using System;
using Dexter.Domain.Model;
using NUnit.Framework;
using SharpTestsEx;

namespace Dexter.Domain.Test.Model {

	[TestFixture]
	public class CategoryTest {
		[Test]
		[Category ( "Builder" )]
		public void CreateNewCategoryShouldReturnAValidInstanceOfCategory ( ) {
			Executing.This ( ( ) => {
			                 	var category = Category.CreateNewCategory ( "MyCategory" , null );
			                 	category.Name.Should ( ).Be.EqualTo ( "MyCategory" );
			                 	category.Id.Should ( ).Be.EqualTo ( 0 );
			                 	category.Parent.Should ( ).Be.Null ( );
			                 } )
				.Should ( )
				.NotThrow ( );
		}

		[Test]
		[Category ( "Builder" )]
		public void CreateNewCategoryShouldThrowArgumentNullExceptionWithNullName ( ) {
			Executing.This ( ( ) => Category.CreateNewCategory ( null , null ) )
				.Should ( )
				.Throw <ArgumentNullException> ( )
				.And
				.Exception
				.ParamName
				.Should ( )
				.Be
				.EqualTo ( "name" );
		}

		[Test]
		[Category ( "Builder" )]
		public void CreateNewCategoryShouldThrowArgumentOutOfRangeExceptionWithParentCategoryWithInvalidID ( ) {
			var parenteCategory = new Category ( "test category" , null );

			Executing.This ( ( ) => Category.CreateNewCategory ( "My Category" , parenteCategory ) )
				.Should ( )
				.Throw <ArgumentOutOfRangeException> ( )
				.And
				.Exception
				.ParamName
				.Should ( )
				.Be
				.EqualTo ( "parent.Id" );
		}

		[Test]
		[Category ( "Validation" )]
		public void ValidateForSaveShouldThrowArgumentOutOfRangeExceptionWithParentCategoryWithInvalidID ( ) {
			var parentCategory = new Category ( "test category" , null );

			Executing.This ( ( ) => {
			                 	var category = new Category ( "MyCategory" , parentCategory );
			                 	category.ValidateForSave ( );
			                 } )
				.Should ( )
				.Throw <ArgumentOutOfRangeException> ( )
				.And
				.Exception
				.ParamName
				.Should ( )
				.Be
				.EqualTo ( "Parent.Id" );
		}

		[Test]
		[Category ( "Validation" )]
		public void ValidateForSaveWithIDPropertyGreatherThenZeroShouldThrownArgumentException ( ) {
			Executing.This ( ( ) => {
			                 	var category = new Category ( "category1" , null ) {
			                 		Id = 2
			                 	};
			                 	category.ValidateForSave ( );
			                 } )
				.Should ( )
				.Throw <ArgumentException> ( )
				.And
				.Exception
				.ParamName
				.Should ( )
				.Be
				.EqualTo ( "Id" );
		}

		[Test]
		[Category ( "Validation" )]
		public void ValidateForSaveWithNullNameShouldThrownArgumentNullException ( ) {
			Executing.This ( ( ) => {
			                 	var category = new Category ( null , null );
			                 	category.ValidateForSave ( );
			                 } )
				.Should ( )
				.Throw <ArgumentNullException> ( )
				.And
				.Exception
				.ParamName
				.Should ( )
				.Be
				.EqualTo ( "Name" );
		}

		[Test]
		[Category ( "Validation" )]
		public void ValidateForUpdateShouldThrowArgumentOutOfRangeExceptionWithParentCategoryWithInvalidID ( ) {
			var parentCategory = new Category ( "test category" , null );

			Executing.This ( ( ) => {
			                 	var category = new Category ( "MyCategory" , parentCategory ) {
			                 		Id = 5
			                 	};
			                 	category.ValidateForUpdate ( );
			                 } )
				.Should ( )
				.Throw <ArgumentOutOfRangeException> ( )
				.And
				.Exception
				.ParamName
				.Should ( )
				.Be
				.EqualTo ( "Parent.Id" );
		}

		[Test]
		[Category ( "Validation" )]
		public void ValidateForUpdateWithIDEqualToZeroShouldThrownArgumentOutOfRangeException ( ) {
			Executing.This ( ( ) => {
			                 	var category = new Category ( "category1" , null );
			                 	category.ValidateForUpdate ( );
			                 } )
				.Should ( )
				.Throw <ArgumentOutOfRangeException> ( )
				.And
				.Exception
				.ParamName
				.Should ( )
				.Be
				.EqualTo ( "Id" );
		}

		[Test]
		[Category ( "Validation" )]
		public void ValidateForUpdateWithNullNameShouldThrownArgumentNullException ( ) {
			Executing.This ( ( ) => {
			                 	var category = new Category ( null , null );
			                 	category.Id = 5;
			                 	category.ValidateForUpdate ( );
			                 } )
				.Should ( )
				.Throw <ArgumentNullException> ( )
				.And
				.Exception
				.ParamName
				.Should ( )
				.Be
				.EqualTo ( "Name" );
		}
	}
}