﻿#region Disclaimer/Info

/////////////////////////////////////////////////////////////////////////////////////////////////
//
//   File:		CommentTest.cs
//   Website:		http://dexterblogengine.com/
//   Authors:		http://dexterblogengine.com/About.ashx
//   Rev:		1
//   Created:		22/11/2010
//   Last edit:		19/01/2011
//   License:		GNU Library General Public License (LGPL)
//   File:            CommentTest.cs
//   For updated news and information please visit http://dexterblogengine.com/
//   Dexter is hosted to Codeplex at http://dexterblogengine.codeplex.com
//   For any question contact info@dexterblogengine.com
//
///////////////////////////////////////////////////////////////////////////////////////////////////

#endregion

using System;
using System.Net.Mail;
using Dexter.Domain.Model;
using Moq;
using NUnit.Framework;
using SharpTestsEx;

namespace Dexter.Domain.Test.Model {
	[TestFixture]
	public class CommentTest {
		[Test]
		[Category ( "Validation" )]
		public void CreateNewCommentWithEmptyEmailShouldThrowArgumentNullException ( ) {
			var itemStub = new Mock<Item>();
			itemStub.Setup(x => x.Id).Returns(1);

			Executing.This ( ( ) => Comment.CreateNewComment ( itemStub.Object , new Uri ( "http://www.tostring.it" ) , null , "message" , "imperugo" , null ) )
				.Should ( )
				.Throw <ArgumentNullException> ( )
				.Exception
				.ParamName
				.Should ( )
				.Be
				.EqualTo ( "email" );
		}

		[Test]
		[Category ( "Validation" )]
		public void CreateNewCommentWithNullEmailShouldThrowArgumentNullException ( ) {
			Executing.This ( ( ) => Comment.CreateNewComment ( new Mock <Item> ( ).Object , new Uri ( "http://www.tostring.it" ) , null , "message" , "imperugo" , null ) )
				.Should ( )
				.Throw <ArgumentNullException> ( )
				.Exception
				.ParamName
				.Should ( )
				.Be
				.EqualTo ( "email" );
		}

		[Test]
		[Category ( "Validation" )]
		public void CreateNewCommentWithNullItemShouldThrowArgumentNullException ( ) {
			Executing.This ( ( ) => Comment.CreateNewComment ( null , new Uri ( "http://www.tostring.it" ) , new MailAddress ( "mail@mydomain.com" ) , "message" , "imperugo" , null ) )
				.Should ( )
				.Throw <ArgumentNullException> ( )
				.Exception
				.ParamName
				.Should ( )
				.Be
				.EqualTo ( "item" );
		}
	}
}