﻿using System;
using System.Configuration;
using Dexter.Core.Data.Repository;
using Dexter.Dependency;
using Dexter.Domain.Model;
using Dexter.Logger.Concretes;
using Dexter.Logger.Contracts;
using Dexter.Repository.NHibernate.Configuration;
using Dexter.Repository.NHibernate.Helpers;
using Dexter.Repository.NHibernate.Test.Helpers;
using Dexter.Services;
using HibernatingRhinos.Profiler.Appender;
using HibernatingRhinos.Profiler.Appender.NHibernate;
using Moq;
using NHibernate;
using NUnit.Framework;
using ISessionFactory = Dexter.Repository.NHibernate.SessionFactory.ISessionFactory;

namespace Dexter.Repository.NHibernate.Test {
	public class BaseNHibernateTest {
		readonly bool createSchemaForEachtest;

		NHibernateHelper nHelper;
		ISessionFactory factory;
		INHibernateConfiguration nHibernateEnvironmentConfiguration;
		Mock<IDexterContainer> stubContainer;
		ISession testSession;

		public ISession TestSession {
			get { return testSession; }
		}

		public BaseNHibernateTest()
		{
			// remember to call the log4net initialization explicitly...Here I go read from the app.config file
			log4net.Config.XmlConfigurator.Configure();
		}

		protected BaseNHibernateTest ( bool createSchemaForEachtest ) : this() {
			this.createSchemaForEachtest = createSchemaForEachtest;
		}

		protected BaseNHibernateTest ( bool createSchemaForEachtest , NHibernateHelper nHelper ) : this() {
			this.createSchemaForEachtest = createSchemaForEachtest;
			this.nHelper = nHelper;
		}

		public ISessionFactory Factory {
			get { return factory; }
		}

		public INHibernateConfiguration NHibernateEnvironmentConfiguration {
			get { return nHibernateEnvironmentConfiguration; }
		}

		protected internal ISession GetNewSession {
			get { return nHelper.GetSession ( ); }
		}

		protected internal global::NHibernate.ISessionFactory SessionFactory {
			get { return nHelper.SessionFactory; }
		}

		protected internal NHibernateHelper NHelper {
			get { return nHelper; }
		}

		[TestFixtureSetUp]
		public virtual void ClassSetup ( ) {
			NHibernateProfiler.Initialize ( );

			stubContainer = new Mock <IDexterContainer> ( );
			stubContainer.Setup ( x => x.Resolve <ILogger> ( ) ).Returns ( NullLogger.Instance );
			DexterContainer.SetCurrent ( stubContainer.Object );
		}

		[TestFixtureTearDown]
		public virtual void ClassCleanup ( ) {
			try {
				ProfilerInfrastructure.FlushAllMessages ( );
				NHibernateProfiler.Stop ( );
			}
			catch { }
		}

		[SetUp]
		public virtual void TestStart ( ) {
			var stubConfiguration = new Mock<INHibernateConfiguration>();

			stubConfiguration.Setup(x => x.IsConfigured).Returns(true);

			stubConfiguration.Setup(x => x.ConnectionString).Returns(ConfigurationManager.ConnectionStrings["dbConn"].ConnectionString);
			stubConfiguration.Setup(x => x.DatabaseSchema).Returns("dbo");
			stubConfiguration.Setup(x => x.DatabaseTablePrefix).Returns("Dexter_");
			stubConfiguration.Setup(x => x.DatabaseType).Returns("MsSql2008");

			stubContainer.Setup ( x => x.Resolve <NHibernateHelper> ( ) ).Returns ( nHelper );
			stubContainer.Setup(x => x.Resolve<INHibernateConfiguration>()).Returns(nHibernateEnvironmentConfiguration);

			nHibernateEnvironmentConfiguration = stubConfiguration.Object;

			// used by the NHibernate custom type DateTimeUtc
			var stubConfigurationRepository = new Mock <IConfigurationService> ( );
			SiteConfiguration siteConfiguration = new SiteConfiguration ( ) {
				TimeZone = TimeZoneInfo.FindSystemTimeZoneById ( "W. Europe Standard Time" )
			};

			stubConfigurationRepository.Setup ( x => x.Configuration ).Returns ( siteConfiguration );

			stubContainer.Setup(x => x.Resolve<IConfigurationService>()).Returns(stubConfigurationRepository.Object);


			if ( nHelper == null ) {
				nHelper = new NHibernateHelper ( nHibernateEnvironmentConfiguration );
				nHelper.Configure ( );
			}

			testSession = nHelper.GetSession ( );

			factory = new DexterSessionFactory ( testSession );

			if ( createSchemaForEachtest ) {
				DropSchema();
				CreateSchema ( );
			}
		}

		[TearDown]
		public virtual void TestComplete ( ) {
			if ( createSchemaForEachtest ) {
				DropSchema ( );
			}
		}

		protected internal void DropSchema ( ) {
			nHelper.DropSchema ( );
		}

		protected internal void CreateSchema ( ) {
			nHelper.InitializeSchema ( );
		}

		protected internal void Persist ( params object[] objs ) {
			using ( ISession session = this.SessionFactory.OpenSession ( ) ) {
				using ( ITransaction tx = session.BeginTransaction ( ) ) {
					foreach ( object o in objs ) {
						session.SaveOrUpdate ( o );
					}

					tx.Commit ( );
				}
			}
		}
	}
}