﻿#region Disclaimer/Info

/////////////////////////////////////////////////////////////////////////////////////////////////
//
//   File:		CommentRepositoryTest.cs
//   Website:		http://dexterblogengine.com/
//   Authors:		http://dexterblogengine.com/About.ashx
//   Rev:		1
//   Created:		19/01/2011
//   Last edit:		19/01/2011
//   License:		GNU Library General Public License (LGPL)
//   File:            CommentRepositoryTest.cs
//   For updated news and information please visit http://dexterblogengine.com/
//   Dexter is hosted to Codeplex at http://dexterblogengine.codeplex.com
//   For any question contact info@dexterblogengine.com
//
///////////////////////////////////////////////////////////////////////////////////////////////////

#endregion

using System;
using Dexter.Logger.Concretes;
using Dexter.Logger.Contracts;
using Dexter.Repository.NHibernate.Repositories;
using NHibernate;
using NUnit.Framework;
using SharpTestsEx;

namespace Dexter.Repository.NHibernate.Test.Repositories
{
	[TestFixture]
	public class CommentRepositoryTest : BaseNHibernateTest
	{
		public CommentRepositoryTest() : base(true) { }

		[Test]
		public void GetApproved_WithBasicItemPropertiesWithNegativePageIndex_ShouldThrowNewArgumentOutOfRangeException()
		{
			this.AddNewBlogRollsToRepository(5);

			Executing.This(() =>
			{
				using (ISession session = TestSession)
				using (ITransaction tx = session.BeginTransaction())
				{
					CommentRepository classUnderTest = new CommentRepository(Factory, NullLogger.Instance);

					classUnderTest.GetApprovedWithBasicItemProperties(-5, 10);
				}
			})
				.Should()
				.Throw<ArgumentOutOfRangeException>()
				.Exception
				.ParamName
				.Should()
				.Be
				.EqualTo("pageIndex");
		}

		[Test]
		public void GetApproved_WithBasicItemPropertiesWithZeroPageIndex_ShouldExecutedWithoutExceptions()
		{
			this.AddNewBlogRollsToRepository(5);

			Executing.This(() =>
			{
				using (ISession session = TestSession)
				using (ITransaction tx = session.BeginTransaction())
				{
					CommentRepository classUnderTest = new CommentRepository(Factory, NullLogger.Instance);

					classUnderTest.GetApprovedWithBasicItemProperties(0, 10);
				}
			})
				.Should()
				.NotThrow();
		}

		[Test]
		public void GetApproved_WithBasicItemPropertiesWithNegativePageSize_ShouldThrowNewArgumentOutOfRangeException()
		{
			this.AddNewBlogRollsToRepository(5);

			Executing.This(() =>
			{
				using (ISession session = TestSession)
				using (ITransaction tx = session.BeginTransaction())
				{
					CommentRepository classUnderTest = new CommentRepository(Factory, NullLogger.Instance);

					classUnderTest.GetApprovedWithBasicItemProperties(1, -10);
				}
			})
				.Should()
				.Throw<ArgumentOutOfRangeException>()
				.Exception
				.ParamName
				.Should()
				.Be
				.EqualTo("pageSize");
		}

		[Test]
		public void GetApproved_WithBasicItemPropertiesWithZeroPageSize_ShouldExecutedWithoutExceptions()
		{
			this.AddNewBlogRollsToRepository(5);
			Executing.This(() =>
			{
				using (ISession session = TestSession)
				using (ITransaction tx = session.BeginTransaction())
				{
					CommentRepository classUnderTest = new CommentRepository(Factory, NullLogger.Instance);

					classUnderTest.GetApprovedWithBasicItemProperties(1, 10);
				}
			})
				.Should()
				.NotThrow();
		}

		[Test]
		public void GetUnapproved_WithBasicItemPropertiesWithNegativePageIndex_ShouldThrowNewArgumentOutOfRangeException()
		{
			this.AddNewBlogRollsToRepository(5);

			Executing.This(() =>
			{
				using (ISession session = TestSession)
				using (ITransaction tx = session.BeginTransaction())
				{
					CommentRepository classUnderTest = new CommentRepository(Factory, NullLogger.Instance);

					classUnderTest.GetUnapprovedWithBasicItemProperties(-5, 10);
				}
			})
				.Should()
				.Throw<ArgumentOutOfRangeException>()
				.Exception
				.ParamName
				.Should()
				.Be
				.EqualTo("pageIndex");
		}

		[Test]
		public void GetUnapproved_WithBasicItemPropertiesWithZeroPageIndex_ShouldExecutedWithoutExceptions()
		{
			this.AddNewBlogRollsToRepository(5);

			Executing.This(() =>
			{
				using (ISession session = TestSession)
				using (ITransaction tx = session.BeginTransaction())
				{
					CommentRepository classUnderTest = new CommentRepository(Factory, NullLogger.Instance);

					classUnderTest.GetUnapprovedWithBasicItemProperties(0, 10);
				}
			})
				.Should()
				.NotThrow();
		}

		[Test]
		public void GetUnapproved_WithBasicItemPropertiesWithNegativePageSize_ShouldThrowNewArgumentOutOfRangeException()
		{
			this.AddNewBlogRollsToRepository(5);

			Executing.This(() =>
			{
				using (ISession session = TestSession)
				using (ITransaction tx = session.BeginTransaction())
				{
					CommentRepository classUnderTest = new CommentRepository(Factory, NullLogger.Instance);

					classUnderTest.GetUnapprovedWithBasicItemProperties(1, -10);
				}
			})
				.Should()
				.Throw<ArgumentOutOfRangeException>()
				.Exception
				.ParamName
				.Should()
				.Be
				.EqualTo("pageSize");
		}

		[Test]
		public void GetUnapproved_WithBasicItemPropertiesWithZeroPageSize_ShouldExecutedWithoutExceptions()
		{
			this.AddNewBlogRollsToRepository(5);
			Executing.This(() =>
			{
				using (ISession session = TestSession)
				using (ITransaction tx = session.BeginTransaction())
				{
					CommentRepository classUnderTest = new CommentRepository(Factory, NullLogger.Instance);

					classUnderTest.GetUnapprovedWithBasicItemProperties(1, 10);
				}
			})
				.Should()
				.NotThrow();
		}

		[Test]
		public void GetRecentCommentWithBasicItemPropertiesWithNegativePageSizeShouldThrowNewArgumentOutOfRangeException()
		{
			this.AddNewBlogRollsToRepository(5);

			Executing.This(() =>
			{
				using (ISession session = TestSession)
				using (ITransaction tx = session.BeginTransaction())
				{
					CommentRepository classUnderTest = new CommentRepository(Factory, NullLogger.Instance);

					classUnderTest.GetRecentCommentWithBasicItemProperties(-10);
				}
			})
				.Should()
				.Throw<ArgumentOutOfRangeException>()
				.Exception
				.ParamName
				.Should()
				.Be
				.EqualTo("number");
		}
	}
}
