﻿#region Disclaimer/Info

/////////////////////////////////////////////////////////////////////////////////////////////////
//
//   File:		ReferrerRepositoryTest.cs
//   Website:		http://dexterblogengine.com/
//   Authors:		http://dexterblogengine.com/About.ashx
//   Rev:		1
//   Created:		19/01/2011
//   Last edit:		19/01/2011
//   License:		GNU Library General Public License (LGPL)
//   File:            ReferrerRepositoryTest.cs
//   For updated news and information please visit http://dexterblogengine.com/
//   Dexter is hosted to Codeplex at http://dexterblogengine.codeplex.com
//   For any question contact info@dexterblogengine.com
//
///////////////////////////////////////////////////////////////////////////////////////////////////

#endregion

using System;
using Dexter.Domain.Model;
using Dexter.Logger.Concretes;
using Dexter.Logger.Contracts;
using Dexter.Repository.NHibernate.Repositories;
using Moq;
using NHibernate;
using NUnit.Framework;
using SharpTestsEx;
using ISessionFactory = Dexter.Repository.NHibernate.SessionFactory.ISessionFactory;

namespace Dexter.Repository.NHibernate.Test.Repositories
{
	[TestFixture]
	public class ReferrerRepositoryTest : BaseNHibernateTest
	{
		public ReferrerRepositoryTest() : base(true) { }

		[Test]
		public void SaveToRepository_WithNullEntity_ShouldThrowNewArgumentException()
		{
			Executing.This(() =>
								{
									using (ISession session = base.TestSession)
									using (ITransaction tx = session.BeginTransaction())
									{
										ReferrerRepository classUnderTest = new ReferrerRepository(base.Factory, NullLogger.Instance);

										classUnderTest.Save(null);

										tx.Commit();
									}
								})
				.Should()
				.Throw<ArgumentNullException>()
				.And
				.Exception
				.ParamName
				.Should()
				.Be
				.EqualTo("item");
		}

		[Test]
		public void SaveToRepository_WithNotNullEntity_ShouldCallValidateMethod()
		{
			var itemToSave = new Mock<Referrer>();
			itemToSave.Setup(x => x.Id).Returns(0);

			var stubFactory = new Mock<ISessionFactory>();
			stubFactory.Setup(x => x.GetSession()).Returns(new Mock<ISession>().Object);

			ReferrerRepository classUnderTest = new ReferrerRepository(stubFactory.Object, NullLogger.Instance);
			try
			{
				classUnderTest.Save(itemToSave.Object);
			}
			catch
			{
				//catch all exception because are not important for the test
				//what happening after the validation method call
			}

			itemToSave.Verify(its => its.ValidateForSave(), Times.Once());
		}

		[Test]
		public void SaveToRepository_WithValidaItem_ShouldPersistCorrectly()
		{
			var item = Referrer.CreateNewReferrer(new Uri("http://dexterblogengine.com"), new Uri("http://www.tostring.it"));

			using (ISession session = base.TestSession)
			using (ITransaction tx = session.BeginTransaction())
			{
				var sut = new ReferrerRepository(base.Factory, NullLogger.Instance);

				sut.Save(item);
				tx.Commit();
			}

			using (var testSession = base.GetNewSession)
			{
				var savedObject = testSession.Get<Referrer>(item.Id);

				savedObject.Should().Not.Be.Null();
			}
		}
	}
}
