﻿#region Disclaimer/Info

/////////////////////////////////////////////////////////////////////////////////////////////////
//
//   File:		RepositoryBaseTest.cs
//   Website:		http://dexterblogengine.com/
//   Authors:		http://dexterblogengine.com/About.ashx
//   Rev:		1
//   Created:		22/11/2010
//   Last edit:		19/01/2011
//   License:		GNU Library General Public License (LGPL)
//   File:            RepositoryBaseTest.cs
//   For updated news and information please visit http://dexterblogengine.com/
//   Dexter is hosted to Codeplex at http://dexterblogengine.codeplex.com
//   For any question contact info@dexterblogengine.com
//
///////////////////////////////////////////////////////////////////////////////////////////////////

#endregion

using System;
using System.Collections.Generic;
using System.Linq;
using Dexter.Domain.Model;
using Dexter.Logger.Concretes;
using Dexter.Logger.Contracts;
using Dexter.Repository.NHibernate.Repositories;
using FizzWare.NBuilder;
using NHibernate;
using NUnit.Framework;
using SharpTestsEx;
using ISessionFactory = Dexter.Repository.NHibernate.SessionFactory.ISessionFactory;

namespace Dexter.Repository.NHibernate.Test.Repositories
{
	[TestFixture]
	public class RepositoryBaseTest : BaseNHibernateTest
	{
		public RepositoryBaseTest() : base(true) { }

		[Test]
		public void Constructor_ShouldNotThrowException()
		{
			using (ITransaction tx = TestSession.BeginTransaction())
			{
				Executing.This(() => { new DerivedFromRepositoryBase(Factory, NullLogger.Instance); })
					.Should()
					.NotThrow();
			}
		}

		[Test]
		public void Count_ShouldReturnTheValidValue()
		{
			IList<PingSite> itemsToSave = Builder<PingSite>
				.CreateListOfSize(3)
				.WhereAll()
				.Have(x => x.Id = 0)
				.Have(x => x.SiteUrl = new Uri("http://www.tostring.it"))
				.Build();

			using (ISession session = GetNewSession)
			using (ITransaction tx = session.BeginTransaction())
			{
				itemsToSave.ForEach(session.SaveOrUpdate);
				tx.Commit();
			}

			DerivedFromRepositoryBase cut = new DerivedFromRepositoryBase(Factory, NullLogger.Instance);
			cut.Count.Should().Be.EqualTo(3);
		}

		[Test]
		public void Clear_ShouldRemoveAllItemsFromTheRepository()
		{
			IList<PingSite> itemsToSave = Builder<PingSite>
				.CreateListOfSize(3)
				.WhereAll()
				.Have(x => x.Id = 0)
				.Have(x => x.SiteUrl = new Uri("http://www.tostring.it"))
				.Build();

			using (ISession session = GetNewSession)
			using (ITransaction tx = session.BeginTransaction())
			{
				itemsToSave.ForEach(session.SaveOrUpdate);
				tx.Commit();
			}

			using (ISession session = Factory.GetSession())
			using (ITransaction tx = session.BeginTransaction())
			{
				DerivedFromRepositoryBase cut = new DerivedFromRepositoryBase(Factory, NullLogger.Instance);
				cut.Clear();

				tx.Commit();
			}

			using (ISession session = GetNewSession)
			using (ITransaction tx = session.BeginTransaction())
			{
				var rowsNumber = session.CreateQuery("select Count(Id) from PingSite")
					.UniqueResult<long>();

				rowsNumber.Should().Be.EqualTo(0);
			}
		}
	}

	internal class DerivedFromRepositoryBase : RepositoryBase<PingSite>
	{
		public DerivedFromRepositoryBase(ISessionFactory sessionFactory, ILogger logger)
			: base(sessionFactory, logger)
		{
		}
	}
}
