﻿#region Disclaimer/Info

/////////////////////////////////////////////////////////////////////////////////////////////////
//
//   File:		TrackBackRepositoryTest.cs
//   Website:		http://dexterblogengine.com/
//   Authors:		http://dexterblogengine.com/About.ashx
//   Rev:		1
//   Created:		22/11/2010
//   Last edit:		19/01/2011
//   License:		GNU Library General Public License (LGPL)
//   File:            TrackBackRepositoryTest.cs
//   For updated news and information please visit http://dexterblogengine.com/
//   Dexter is hosted to Codeplex at http://dexterblogengine.codeplex.com
//   For any question contact info@dexterblogengine.com
//
///////////////////////////////////////////////////////////////////////////////////////////////////

#endregion

using System;
using Dexter.Domain.Model;
using Dexter.Logger.Concretes;
using Dexter.Logger.Contracts;
using Dexter.Repository.NHibernate.Repositories;
using Moq;
using NHibernate;
using NUnit.Framework;
using SharpTestsEx;
using ISessionFactory = Dexter.Repository.NHibernate.SessionFactory.ISessionFactory;

namespace Dexter.Repository.NHibernate.Test.Repositories
{
	[TestFixture]
	public class TrackBackRepositoryTest : BaseNHibernateTest
	{
		public TrackBackRepositoryTest() : base(true) { }

		[Test]
		public void SaveToRepository_WithNullEntity_ShouldThrowNewArgumentException()
		{
			Executing.This(() =>
			{
				using (ISession session = TestSession)
				using (ITransaction tx = session.BeginTransaction())
				{
					TrackBackRepository classUnderTest = new TrackBackRepository(Factory, NullLogger.Instance);

					classUnderTest.Save(null);

					tx.Commit();
				}
			})
				.Should()
				.Throw<ArgumentNullException>()
				.And
				.Exception
				.ParamName
				.Should()
				.Be
				.EqualTo("item");
		}

		[Test]
		public void SaveToRepository_WithNotNullEntity_ShouldCallValidateMethod()
		{
			var itemToSave = new Mock<TrackBack>();
			itemToSave.Setup(x => x.Name).Returns("tostring.it");
			itemToSave.Setup(x => x.Url).Returns(new Uri("http://www.tostring.it"));
			itemToSave.Setup(x => x.Id).Returns(0);

			var stubFactory = new Mock<ISessionFactory>();
			stubFactory.Setup(x => x.GetSession()).Returns(new Mock<ISession>().Object);

			TrackBackRepository classUnderTest = new TrackBackRepository(stubFactory.Object, NullLogger.Instance);
			try
			{
				classUnderTest.Save(itemToSave.Object);
			}
			catch
			{
				//catch all exception because are not important for the test
				//what happening after the validation method call
			}

			itemToSave.Verify(its => its.ValidateForSave(),Times.Once());
		}
	}
}
