﻿#region Disclaimer/Info

/////////////////////////////////////////////////////////////////////////////////////////////////
//
//   File:		UserRepositoryTest.cs
//   Website:		http://dexterblogengine.com/
//   Authors:		http://dexterblogengine.com/About.ashx
//   Rev:		1
//   Created:		22/11/2010
//   Last edit:		19/01/2011
//   License:		GNU Library General Public License (LGPL)
//   File:            UserRepositoryTest.cs
//   For updated news and information please visit http://dexterblogengine.com/
//   Dexter is hosted to Codeplex at http://dexterblogengine.codeplex.com
//   For any question contact info@dexterblogengine.com
//
///////////////////////////////////////////////////////////////////////////////////////////////////

#endregion

using System;
using Dexter.Domain.Model;
using Dexter.Logger.Concretes;
using Dexter.Logger.Contracts;
using Dexter.Repository.NHibernate.Repositories;
using Moq;
using NHibernate;
using NUnit.Framework;
using SharpTestsEx;
using ISessionFactory = Dexter.Repository.NHibernate.SessionFactory.ISessionFactory;

namespace Dexter.Repository.NHibernate.Test.Repositories
{
	[TestFixture]
	public class UserRepositoryTest : BaseNHibernateTest
	{
		[Test]
		public void SaveToRepository_WithNullEntity_ShouldThrowNewArgumentException()
		{
			Executing.This(() =>
								{
									using (ISession session = TestSession)
									using (ITransaction tx = session.BeginTransaction())
									{
										UserRepository classUnderTest = new UserRepository(Factory, NullLogger.Instance);

										classUnderTest.Save(null);

										tx.Commit();
									}
								})
				.Should()
				.Throw<ArgumentNullException>()
				.And
				.Exception
				.ParamName
				.Should()
				.Be
				.EqualTo("item");
		}

		[Test]
		public void UpdateToRepository_WithNullEntity_ShouldThrowNewArgumentException()
		{
			Executing.This(() =>
			{
				using (ISession session = TestSession)
				using (ITransaction tx = session.BeginTransaction())
				{
					UserRepository classUnderTest = new UserRepository(Factory, NullLogger.Instance);

					classUnderTest.Update(null);

					tx.Commit();
				}
			})
				.Should()
				.Throw<ArgumentNullException>()
				.And
				.Exception
				.ParamName
				.Should()
				.Be
				.EqualTo("item");
		}

		[Test]
		public void DeleteToRepository_WithNullEntity_ShouldThrowNewArgumentException()
		{
			Executing.This(() =>
			{
				using (ISession session = TestSession)
				using (ITransaction tx = session.BeginTransaction())
				{
					UserRepository classUnderTest = new UserRepository(Factory, NullLogger.Instance);

					classUnderTest.Delete(null);

					tx.Commit();
				}
			})
				.Should()
				.Throw<ArgumentNullException>()
				.And
				.Exception
				.ParamName
				.Should()
				.Be
				.EqualTo("item");
		}

		[Test]
		public void SaveToRepository_WithNotNullEntity_ShouldCallValidateMethod()
		{
			var itemToSave = new Mock<User>();
			itemToSave.Setup(x => x.Id).Returns(0);

			var stubFactory = new Mock<ISessionFactory>();
			stubFactory.Setup(x => x.GetSession()).Returns(new Mock<ISession>().Object);

			UserRepository classUnderTest = new UserRepository(stubFactory.Object, NullLogger.Instance);
			try
			{
				classUnderTest.Save(itemToSave.Object);
			}
			catch
			{
				//catch all exception because are not important for the test
				//what happening after the validation method call
			}

			itemToSave.Verify(its => its.ValidateForSave(), Times.Once());
		}

		[Test]
		public void UpdateToRepository_WithNotNullEntity_ShouldCallValidateMethod()
		{
			var itemToSave = new Mock<User>();
			itemToSave.Setup(x => x.Id).Returns(0);

			var stubFactory = new Mock<ISessionFactory>();
			stubFactory.Setup(x => x.GetSession()).Returns(new Mock<ISession>().Object);

			UserRepository classUnderTest = new UserRepository(stubFactory.Object, NullLogger.Instance);
			try
			{
				classUnderTest.Update(itemToSave.Object);
			}
			catch
			{
				//catch all exception because are not important for the test
				//what happening after the validation method call
			}

			itemToSave.Verify(its => its.ValidateForUpdate(),Times.Once());
		}

		[Test]
		public void DeleteToRepository_WithNotNullEntity_ShouldCallValidateMethod()
		{
			var itemToSave = new Mock<User>();
			itemToSave.Setup(x => x.Id).Returns(0);

			var stubFactory = new Mock<ISessionFactory>();
			stubFactory.Setup(x => x.GetSession()).Returns(new Mock<ISession>().Object);

			UserRepository classUnderTest = new UserRepository(stubFactory.Object, NullLogger.Instance);
			try
			{
				classUnderTest.Delete(itemToSave.Object);
			}
			catch
			{
				//catch all exception because are not important for the test
				//what happening after the validation method call
			}

			itemToSave.Verify(its => its.ValidateForDelete(), Times.Once());
		}
	}
}
