﻿ #region Disclaimer/Info
 
 /////////////////////////////////////////////////////////////////////////////////////////////////
 //
 //   File:		CategoryServicetest.cs
 //   Website:		http://dexterblogengine.com/
 //   Authors:		http://dexterblogengine.com/About.ashx
 //   Rev:		1
 //   Created:		28/11/2010
 //   Last edit:		19/01/2011
 //   License:		GNU Library General Public License (LGPL)
 //   File:            CategoryServicetest.cs
 //   For updated news and information please visit http://dexterblogengine.com/
 //   Dexter is hosted to Codeplex at http://dexterblogengine.codeplex.com
 //   For any question contact info@dexterblogengine.com
 //
 ///////////////////////////////////////////////////////////////////////////////////////////////////
 
 #endregion

using Dexter.Caching;
using Dexter.Logger.Contracts;
using Dexter.Repository;
using NUnit.Framework;
using Rhino.Mocks;
using SharpTestsEx;

namespace Dexter.Services.Implementation.Test
{
	[TestFixture]
	public class CategoryServiceTest
	{
		ILogger stubLogger;
		ICacheProvider stubCacheProvider;

		[TestFixtureSetUp]
		public void FixtureSetUp ( ) {
			stubLogger = MockRepository.GenerateStub<ILogger> ( );
			stubCacheProvider = MockRepository.GenerateStub<ICacheProvider> ( );
		}

		[Test]
		public void Save_WithValidParameters_ShouldInvokeCategorySaving()
		{
			bool actual = false;

			var stub = MockRepository.GenerateStub<ICategoryRepository>();
			var sut = new CategoryService ( stub, stubLogger, stubCacheProvider );

			stub.Expect(x => x.Save(null));

			sut.CategorySaving += (s, e) => actual = true;

			sut.Save(null);

			actual.Should().Be.True();
		}

		[Test]
		public void Save_WithValidParametersAndCancelSetToTrue_ShouldNotInvokeSaveMethodIntoTheRepositoryMock()
		{
			var mock = MockRepository.GenerateMock<ICategoryRepository>();
			var sut = new CategoryService ( mock, stubLogger, stubCacheProvider );

			mock.Expect(x => x.Save(null));

			sut.CategorySaving += (s, e) => e.Cancel = true;

			sut.Save(null);

			mock.AssertWasNotCalled ( x => x.Save ( null ), a => a.IgnoreArguments (  ) );
		}

		[Test]
		public void Save_WithValidParameters_ShouldInvokeSaveMethodIntoRepositoryMock()
		{
			var mock = MockRepository.GenerateMock<ICategoryRepository>();
			var sut = new CategoryService ( mock, stubLogger, stubCacheProvider );

			sut.Save(null);

			mock.AssertWasCalled ( x => x.Save ( null ), a => a.IgnoreArguments (  ) );
		}

		[Test]
		public void Save_WithValidParameters_ShouldInvokeCategorySaved()
		{
			bool actual = false;

			var stub = MockRepository.GenerateStub<ICategoryRepository>();
			var sut = new CategoryService ( stub, stubLogger, stubCacheProvider );

			stub.Expect(x => x.Save(null));

			sut.CategorySaved += (s, e) => actual = true;

			sut.Save(null);

			actual.Should().Be.True();
		}

		[Test]
		public void Update_WithValidParameters_ShouldInvokeCategoryUpdating()
		{
			bool actual = false;

			var stub = MockRepository.GenerateStub<ICategoryRepository>();
			var sut = new CategoryService ( stub, stubLogger, stubCacheProvider );

			stub.Expect(x => x.Update(null));

			sut.CategoryUpdating += (s, e) => actual = true;

			sut.Update(null);

			actual.Should().Be.True();
		}

		[Test]
		public void Update_WithValidParametersAndCancelSetToTrue_ShouldNotInvokeSaveMethodIntoTheRepositoryMock()
		{
			var mock = MockRepository.GenerateMock<ICategoryRepository>();
			var sut = new CategoryService ( mock, stubLogger, stubCacheProvider );

			mock.Expect(x => x.Update(null));

			sut.CategoryUpdating += (s, e) => e.Cancel = true;

			sut.Update(null);

			mock.AssertWasNotCalled(x => x.Update(null), a => a.IgnoreArguments());
		}

		[Test]
		public void Update_WithValidParameters_ShouldInvokeSaveMethodIntoRepositoryMock()
		{
			var mock = MockRepository.GenerateMock<ICategoryRepository>();
			var sut = new CategoryService ( mock, stubLogger, stubCacheProvider );

			sut.Update(null);

			mock.AssertWasCalled(x => x.Update(null), a => a.IgnoreArguments());
		}

		[Test]
		public void Update_WithValidParameters_ShouldInvokeCategoryUpdated()
		{
			bool actual = false;

			var stub = MockRepository.GenerateStub<ICategoryRepository>();
			var sut = new CategoryService ( stub, stubLogger, stubCacheProvider );

			stub.Expect(x => x.Update(null));

			sut.CategoryUpdated += (s, e) => actual = true;

			sut.Update(null);

			actual.Should().Be.True();
		}

		[Test]
		public void DeleteCategory_WithValidParameters_ShouldInvokeCategoryDeleting()
		{
			bool actual = false;

			var stub = MockRepository.GenerateStub<ICategoryRepository>();
			var sut = new CategoryService ( stub, stubLogger, stubCacheProvider );

			stub.Expect(x => x.DeleteCategory(null, null));

			sut.CategoryDeleting += (s, e) => actual = true;

			sut.DeleteCategory(null,null);

			actual.Should().Be.True();
		}

		[Test]
		public void DeleteCategory_WithValidParametersAndCancelSetToTrue_ShouldNotInvokeSaveMethodIntoTheRepositoryMock()
		{
			var mock = MockRepository.GenerateMock<ICategoryRepository>();
			var sut = new CategoryService ( mock, stubLogger, stubCacheProvider );

			mock.Expect(x => x.DeleteCategory(null, null));

			sut.CategoryDeleting += (s, e) => e.Cancel = true;

			sut.DeleteCategory(null, null);

			mock.AssertWasNotCalled(x => x.DeleteCategory(null,null), a => a.IgnoreArguments());
		}

		[Test]
		public void DeleteCategory_WithValidParameters_ShouldInvokeSaveMethodIntoRepositoryMock()
		{
			var mock = MockRepository.GenerateMock<ICategoryRepository>();
			var sut = new CategoryService ( mock, stubLogger, stubCacheProvider );

			sut.DeleteCategory(null,null);

			mock.AssertWasCalled(x => x.DeleteCategory(null,null), a => a.IgnoreArguments());
		}

		[Test]
		public void DeleteCategory_WithValidParameters_ShouldInvokeCategoryDeleted()
		{
			bool actual = false;

			var stub = MockRepository.GenerateStub<ICategoryRepository>();
			var sut = new CategoryService ( stub, stubLogger, stubCacheProvider );

			stub.Expect(x => x.DeleteCategory(null,null));

			sut.CategoryDeleted += (s, e) => actual = true;

			sut.DeleteCategory(null,null);

			actual.Should().Be.True();
		}

		[Test]
		public void GetCategoriesStructure_WithValidParameters_ShouldInvokeCategoriesRetrieved ( )
		{
			bool actual = false;

			var stub = MockRepository.GenerateStub<ICategoryRepository>();
			var sut = new CategoryService ( stub, stubLogger, stubCacheProvider );

			stub.Expect(x => x.GetCategoriesStructure());

			sut.CategoriesRetrieved += (s, e) => actual = true;

			sut.GetCategoriesStructure();

			actual.Should().Be.True();
		}

		[Test]
		public void GetDefaultCategoryWithChilds_WithValidParameters_ShouldInvokeDefaultCategoryRetrieved()
		{
			bool actual = false;

			var stub = MockRepository.GenerateStub<ICategoryRepository>();
			var sut = new CategoryService ( stub, stubLogger, stubCacheProvider );

			stub.Expect(x => x.GetDefaultCategoryWithChilds());

			sut.DefaultCategoryRetrieved += (s, e) => actual = true;

			sut.GetDefaultCategoryWithChilds();

			actual.Should().Be.True();
		}

		[Test]
		public void GetCategoryByKeyWithChilds_WithValidParameters_ShouldInvokeDefaultCategoryRetrieved()
		{
			bool actual = false;

			var stub = MockRepository.GenerateStub<ICategoryRepository>();
			var sut = new CategoryService ( stub, stubLogger, stubCacheProvider );

			stub.Expect(x => x.GetCategoryByKeyWithChilds(1));

			sut.DefaultCategoryRetrieved += (s, e) => actual = true;

			sut.GetCategoryByKeyWithChilds(1);

			actual.Should().Be.True();
		}
	}
}
