﻿ #region Disclaimer/Info
 
 /////////////////////////////////////////////////////////////////////////////////////////////////
 //
 //   File:		CommentServiceTest.cs
 //   Website:		http://dexterblogengine.com/
 //   Authors:		http://dexterblogengine.com/About.ashx
 //   Rev:		1
 //   Created:		04/01/2011
 //   Last edit:		19/01/2011
 //   License:		GNU Library General Public License (LGPL)
 //   File:            CommentServiceTest.cs
 //   For updated news and information please visit http://dexterblogengine.com/
 //   Dexter is hosted to Codeplex at http://dexterblogengine.codeplex.com
 //   For any question contact info@dexterblogengine.com
 //
 ///////////////////////////////////////////////////////////////////////////////////////////////////
 
 #endregion

using Dexter.Caching;
using Dexter.Logger.Contracts;
using Dexter.Repository;
using NUnit.Framework;
using Rhino.Mocks;
using SharpTestsEx;

namespace Dexter.Services.Implementation.Test {
	[TestFixture]
	public class CommentServiceTest  {
		ILogger stubLogger;
		ICacheProvider stubCacheProvider;

		[TestFixtureSetUp]
		public void FixtureSetUp ( ) {
			stubLogger = MockRepository.GenerateStub<ILogger> ( );
			stubCacheProvider = MockRepository.GenerateStub<ICacheProvider> ( );
		}

		[Test]
		public void GetRecent_Should_Invoke_SlugRetrieved ( ) {
			bool actual = false;

			var mock = MockRepository.GenerateMock<ICommentRepository> ( );
			var postrepositoryStup = MockRepository.GenerateStub<IPostRepository> ( );

			ICommentService sut = new CommentService ( mock, postrepositoryStup, stubLogger, MockRepository.GenerateStub<IConfigurationService> ( ), stubCacheProvider );

			sut.RecentCommentRetrieved += ( s, e ) => actual = true;

			sut.GetRecent ( 10 );

			actual.Should ( ).Be.True ( );
		}

		[Test]
		public void GetRecent_Should_Invoke_SlugRetrieving_Before_SlugRetrieved ( ) {
			int actual = 0;

			var mock = MockRepository.GenerateMock<ICommentRepository> ( );
			var postrepositoryStup = MockRepository.GenerateStub<IPostRepository> ( );

			ICommentService sut = new CommentService ( mock, postrepositoryStup, stubLogger, MockRepository.GenerateStub<IConfigurationService> ( ), stubCacheProvider );

			sut.RecentCommentRetrieving += ( s, e ) => actual = 1;
			sut.RecentCommentRetrieved += ( s, e ) => actual = actual - 1;

			sut.GetRecent ( 10 );

			actual.Should ( ).Be.EqualTo ( 0 );
		}

		[Test]
		public void GetRecent_ShouldInvokeRepositoryMethod ( ) {
			var mock = MockRepository.GenerateMock<ICommentRepository> ( );
			var postrepositoryStup = MockRepository.GenerateStub<IPostRepository> ( );

			ICommentService sut = new CommentService ( mock, postrepositoryStup, stubLogger, MockRepository.GenerateStub<IConfigurationService> ( ), stubCacheProvider );

			mock.Expect ( x => x.GetRecentCommentWithBasicItemProperties ( 10 , false ) ).IgnoreArguments ( ).Return ( null );

			var result = sut.GetRecent ( 10 );

			mock.VerifyAllExpectations ( );
		}
	}
}
