﻿ #region Disclaimer/Info
 
 /////////////////////////////////////////////////////////////////////////////////////////////////
 //
 //   File:		ConfigurationServiceTest.cs
 //   Website:		http://dexterblogengine.com/
 //   Authors:		http://dexterblogengine.com/About.ashx
 //   Rev:		1
 //   Created:		29/11/2010
 //   Last edit:		19/01/2011
 //   License:		GNU Library General Public License (LGPL)
 // 
 //   For updated news and information please visit http://dexterblogengine.com/
 //   Dexter is hosted to Codeplex at http://dexterblogengine.codeplex.com
 //   For any question contact info@dexterblogengine.com
 //
 ///////////////////////////////////////////////////////////////////////////////////////////////////
 
 #endregion

using System;
using Dexter.Caching;
using Dexter.Dependency;
using Dexter.Domain.Model;
using Dexter.Logger.Concretes;
using Dexter.Logger.Contracts;
using Dexter.Repository;
using Dexter.Services.Implementation.Test.Helpers;
using NUnit.Framework;
using Rhino.Mocks;
using SharpTestsEx;

namespace Dexter.Services.Implementation.Test {
	[TestFixture]
	public class ConfigurationServiceTest {
		ILogger stubLogger;

		[TestFixtureSetUp]
		public void FixtureSetUp ( ) {
			stubLogger = MockRepository.GenerateStub<ILogger> ( );
		}


		[Test]
		public void GetConfiguration_WithCachedItem_ShouldNotCallTheRepository ( ) {
			var configuration = ConfigurationHelper.GetNewConfiguration ( );

			var mock = MockRepository.GenerateMock <IConfigurationRepository> ( );
			var cacheStub = MockRepository.GenerateStub <ICacheProvider> ( );
			cacheStub.Expect ( x => x.Get <SiteConfiguration> ( "" ) ).IgnoreArguments ( ).Return ( configuration );

			var sut = new ConfigurationService ( cacheStub , mock , stubLogger );

			var expectedConfiguration = sut.Configuration;

			expectedConfiguration.BlogName.Should ( )
				.Be
				.EqualTo ( configuration.BlogName );

			mock.AssertWasNotCalled ( x => x.GetConfiguration ( ) );
		}

		[Test]
		public void GetConfiguration_WithNoCachedItem_ShouldCallTheRepository ( ) {
			var configuration = ConfigurationHelper.GetNewConfiguration ( );

			var mock = MockRepository.GenerateMock <IConfigurationRepository> ( );
			var cacheStub = MockRepository.GenerateStub <ICacheProvider> ( );
			cacheStub.Expect ( x => x.Get <SiteConfiguration> ( "" ) ).IgnoreArguments ( ).Return ( null );

			var sut = new ConfigurationService ( cacheStub , mock , stubLogger );

			var returnConf = new Configuration {SerializedConfiguration = configuration.Serialize ( )};

			mock.Stub ( x => x.GetConfiguration ( ) ).Return ( returnConf );

			var expectedConfiguration = sut.Configuration;

			expectedConfiguration.BlogName.Should ( )
				.Be
				.EqualTo ( configuration.BlogName );

			mock.AssertWasCalled ( x => x.GetConfiguration ( ) );
		}
	}
}
