﻿ #region Disclaimer/Info
 
 /////////////////////////////////////////////////////////////////////////////////////////////////
 //
 //   File:		PageServiceTest.cs
 //   Website:		http://dexterblogengine.com/
 //   Authors:		http://dexterblogengine.com/About.ashx
 //   Rev:		1
 //   Created:		31/12/2010
 //   Last edit:		19/01/2011
 //   License:		GNU Library General Public License (LGPL)
 //   File:            PageServiceTest.cs
 //   For updated news and information please visit http://dexterblogengine.com/
 //   Dexter is hosted to Codeplex at http://dexterblogengine.codeplex.com
 //   For any question contact info@dexterblogengine.com
 //
 ///////////////////////////////////////////////////////////////////////////////////////////////////
 
 #endregion

using System.Collections.Generic;
using Dexter.Caching;
using Dexter.Logger.Contracts;
using Dexter.Repository;
using NUnit.Framework;
using Rhino.Mocks;
using SharpTestsEx;

namespace Dexter.Services.Implementation.Test {
	[TestFixture]
	public class PageServiceTest {
		ILogger stubLogger;
		ICacheProvider stubCacheProvider;

		[TestFixtureSetUp]
		public void FixtureSetUp ( ) {
			stubLogger = MockRepository.GenerateStub<ILogger> ( );
			stubCacheProvider = MockRepository.GenerateStub<ICacheProvider> ( );
		}

		[Test]
		public void GetAllSlug_Should_Invoke_SlugRetrieved ( ) {
			bool actual = false;

			var mock = MockRepository.GenerateMock<IPageRepository> ( );

			IPageService sut = new PageService ( mock, stubLogger, stubCacheProvider );

			//mock.Expect(x => x.GetAllPagesSlugs());

			sut.SlugRetrieved += ( s, e ) => actual = true;

			sut.GetAllSlugs ( );

			actual.Should ( ).Be.True ( );
		}

		[Test]
		public void GetAllSlug_Should_Invoke_SlugRetrieving_Before_SlugRetrieved ( ) {
			int actual = 0;

			var mock = MockRepository.GenerateMock<IPageRepository> ( );

			IPageService sut = new PageService ( mock, stubLogger, stubCacheProvider );

			//mock.Expect(x => x.GetAllPagesSlugs());

			sut.SlugRetrieving += ( s, e ) => actual = 1;
			sut.SlugRetrieved += ( s, e ) => actual = actual - 1;

			sut.GetAllSlugs ( );

			actual.Should ( ).Be.EqualTo ( 0 );
		}

		[Test]
		public void GetAllSlug_ShouldInvokeRepositoryMethod ( ) {
			var mock = MockRepository.GenerateMock<IPageRepository> ( );

			IPageService sut = new PageService ( mock, stubLogger, stubCacheProvider );

			mock.Expect ( x => x.GetAllPagesSlugs ( ) ).Return ( null );

			IEnumerable<string> allSlugs = sut.GetAllSlugs ( );

			mock.VerifyAllExpectations ( );
		}

	}
}
