﻿#region Disclaimer/Info

/////////////////////////////////////////////////////////////////////////////////////////////////
//
//   File:		PostServiceTest.cs
//   Website:		http://dexterblogengine.com/
//   Authors:		http://dexterblogengine.com/About.ashx
//   Rev:		1
//   Created:		28/12/2010
//   Last edit:		19/01/2011
//   License:		GNU Library General Public License (LGPL)
//   File:            PostServiceTest.cs
//   For updated news and information please visit http://dexterblogengine.com/
//   Dexter is hosted to Codeplex at http://dexterblogengine.codeplex.com
//   For any question contact info@dexterblogengine.com
//
///////////////////////////////////////////////////////////////////////////////////////////////////

#endregion

using Dexter.Caching;
using Dexter.Logger.Contracts;
using Dexter.Repository;
using NUnit.Framework;
using Rhino.Mocks;
using SharpTestsEx;

namespace Dexter.Services.Implementation.Test {
	[TestFixture]
	public class PostServiceTest {

		ILogger stubLogger;
		ICacheProvider stubCacheProvider;

		[TestFixtureSetUp]
		public void FixtureSetUp ( ) {
			stubLogger = MockRepository.GenerateStub<ILogger> ( );
			stubCacheProvider = MockRepository.GenerateStub<ICacheProvider> ( );
		}

		[Test]
		public void GetByKey_WithValidParameters_ShouldInvokePostRetrievedByKey ( ) {
			bool actual = false;
			var parameter = 1;

			var stub = MockRepository.GenerateStub <IPostRepository> ( );
			var stubCategoryRepository = MockRepository.GenerateMock <ICategoryRepository> ( );
			IPostService sut = new PostService ( stubLogger, stub, stubCategoryRepository, stubCacheProvider );

			sut.PostRetrievedByKey += ( s , e ) => {
			                          	actual = true;
			                          	e.Parameter.Should ( ).Be.EqualTo ( parameter );
			                          };

			sut.GetByKey ( parameter );

			actual.Should ( ).Be.True ( );
		}

		[Test]
		public void GetByKey_WithValidParameters_ShouldInvokePostRetrievingByKey ( ) {
			bool actual = false;
			var parameter = 1;

			var stub = MockRepository.GenerateStub <IPostRepository> ( );
			var stubCategoryRepository = MockRepository.GenerateMock <ICategoryRepository> ( );
			IPostService sut = new PostService ( stubLogger, stub, stubCategoryRepository, stubCacheProvider );

			sut.PostRetrievingByKey += ( s , e ) => {
			                           	actual = true;
			                           	e.Parameter.Should ( ).Be.EqualTo ( parameter );
			                           	e.Cancel = true;
			                           };

			sut.GetByKey ( parameter );

			actual.Should ( ).Be.True ( );
		}

		[Test]
		public void GetByKey_WithValidParameters_ShouldInvokeRepositoryMethod ( ) {
			var parameter = 1;

			var mock = MockRepository.GenerateMock <IPostRepository> ( );
			var stubCategoryRepository = MockRepository.GenerateMock <ICategoryRepository> ( );
			IPostService sut = new PostService ( stubLogger, mock, stubCategoryRepository, stubCacheProvider );

			mock.Expect ( x => x.GetByKey ( parameter , false ) ).Return ( null );

			sut.GetByKey ( parameter );

			mock.VerifyAllExpectations ( );
		}

		[Test]
		public void GetByMonth_WithValidParameters_ShouldInvokePostsPublishedRetrieving ( ) {
			bool actual = false;

			var stub = MockRepository.GenerateStub <IPostRepository> ( );
			var stubCategoryRepository = MockRepository.GenerateMock <ICategoryRepository> ( );
			IPostService sut = new PostService ( stubLogger, stub, stubCategoryRepository, stubCacheProvider );

			sut.PostsRetrievingByMonth += ( s , e ) => {
			                              	actual = true;
			                              	e.Cancel = true;
			                              };

			sut.GetByMonth ( 2 , 2010 , 1 , 10 );

			actual.Should ( ).Be.True ( );
		}

		[Test]
		public void GetByMonth_WithValidParameters_ShouldInvokeRepositoryMethod ( ) {
			var mock = MockRepository.GenerateMock <IPostRepository> ( );
			var stubCategoryRepository = MockRepository.GenerateMock <ICategoryRepository> ( );
			IPostService sut = new PostService ( stubLogger, mock, stubCategoryRepository, stubCacheProvider );

			mock.Expect ( x => x.GetByMonth ( 2 , 2010 , 1 , 10 , true , false ) ).Return ( null );

			sut.GetByMonth ( 2 , 2010 , 1 , 10 );

			mock.VerifyAllExpectations ( );
		}

		[Test]
		public void GetBySlug_WithValidParameters_ShouldInvokePostRetrievedByKey ( ) {
			bool actual = false;
			var parameter = "slug";

			var stub = MockRepository.GenerateStub <IPostRepository> ( );
			var stubCategoryRepository = MockRepository.GenerateMock <ICategoryRepository> ( );
			IPostService sut = new PostService ( stubLogger, stub, stubCategoryRepository, stubCacheProvider );

			sut.PostRetrievedBySlug += ( s , e ) => {
			                           	actual = true;
			                           	e.Parameter.Should ( ).Be.EqualTo ( parameter );
			                           };

			sut.GetBySlug ( parameter );

			actual.Should ( ).Be.True ( );
		}

		[Test]
		public void GetBySlug_WithValidParameters_ShouldInvokePostRetrievingByKey ( ) {
			bool actual = false;
			var parameter = "slug";

			var stub = MockRepository.GenerateStub <IPostRepository> ( );
			var stubCategoryRepository = MockRepository.GenerateMock <ICategoryRepository> ( );
			IPostService sut = new PostService ( stubLogger, stub, stubCategoryRepository, stubCacheProvider );

			sut.PostRetrievingBySlug += ( s , e ) => {
			                            	actual = true;
			                            	e.Parameter.Should ( ).Be.EqualTo ( parameter );
			                            	e.Cancel = true;
			                            };

			sut.GetBySlug ( parameter );

			actual.Should ( ).Be.True ( );
		}

		[Test]
		public void GetBySlug_WithValidParameters_ShouldInvokeRepositoryMethod ( ) {
			var parameter = "slug";

			var mock = MockRepository.GenerateMock <IPostRepository> ( );
			var stubCategoryRepository = MockRepository.GenerateMock <ICategoryRepository> ( );
			IPostService sut = new PostService ( stubLogger, mock, stubCategoryRepository, stubCacheProvider );

			mock.Expect ( x => x.GetBySlug ( parameter , false ) ).Return ( null );

			sut.GetBySlug ( parameter );

			mock.VerifyAllExpectations ( );
		}

		[Test]
		public void GetCompleteList_WithValidParameters_ShouldInvokePostsRetrieved ( ) {
			bool actual = false;

			var stub = MockRepository.GenerateStub <IPostRepository> ( );
			var stubCategoryRepository = MockRepository.GenerateMock <ICategoryRepository> ( );
			IPostService sut = new PostService ( stubLogger, stub, stubCategoryRepository, stubCacheProvider );

			sut.PostsRetrieved += ( s , e ) => { actual = true; };

			sut.GetCompleteList ( 1 , 10 );

			actual.Should ( ).Be.True ( );
		}

		[Test]
		public void GetCompleteList_WithValidParameters_ShouldInvokePostsRetrieving ( ) {
			bool actual = false;

			var stub = MockRepository.GenerateStub <IPostRepository> ( );
			var stubCategoryRepository = MockRepository.GenerateMock <ICategoryRepository> ( );
			IPostService sut = new PostService ( stubLogger, stub, stubCategoryRepository, stubCacheProvider );

			sut.PostsRetrieving += ( s , e ) => {
			                       	actual = true;
			                       	e.Cancel = true;
			                       };

			sut.GetCompleteList ( 1 , 10 );

			actual.Should ( ).Be.True ( );
		}

		[Test]
		public void GetCompleteList_WithValidParameters_ShouldInvokeRepositoryMethod ( ) {
			var mock = MockRepository.GenerateMock <IPostRepository> ( );
			var stubCategoryRepository = MockRepository.GenerateMock <ICategoryRepository> ( );
			IPostService sut = new PostService ( stubLogger, mock, stubCategoryRepository, stubCacheProvider );

			mock.Expect ( x => x.GetCompleteList ( 1 , 10 , false ) ).Return ( null );

			sut.GetCompleteList ( 1 , 10 );

			mock.VerifyAllExpectations ( );
		}

		[Test]
		public void GetMonths_WithValidParameters_ShouldInvokePostsRecentRetrieved ( ) {
			bool actual = false;

			var stub = MockRepository.GenerateStub <IPostRepository> ( );
			var stubCategoryRepository = MockRepository.GenerateMock <ICategoryRepository> ( );
			IPostService sut = new PostService ( stubLogger, stub, stubCategoryRepository, stubCacheProvider );

			sut.MonthRetrieved += ( s , e ) => { actual = true; };

			sut.GetMonths ( );

			actual.Should ( ).Be.True ( );
		}

		[Test]
		public void GetMonths_WithValidParameters_ShouldInvokePostsRecentRetrieving ( ) {
			bool actual = false;

			var stub = MockRepository.GenerateStub <IPostRepository> ( );
			var stubCategoryRepository = MockRepository.GenerateMock <ICategoryRepository> ( );
			IPostService sut = new PostService ( stubLogger, stub, stubCategoryRepository, stubCacheProvider );

			sut.MonthRetrieving += ( s , e ) => {
			                       	actual = true;
			                       	e.Cancel = true;
			                       };

			sut.GetMonths ( );

			actual.Should ( ).Be.True ( );
		}

		[Test]
		public void GetMonths_WithValidParameters_ShouldInvokeRepositoryMethod ( ) {
			var mock = MockRepository.GenerateMock <IPostRepository> ( );
			var stubCategoryRepository = MockRepository.GenerateMock <ICategoryRepository> ( );
			IPostService sut = new PostService ( stubLogger, mock, stubCategoryRepository, stubCacheProvider );

			mock.Expect ( x => x.GetMonths ( ) ).Return ( null );

			sut.GetMonths ( );

			mock.VerifyAllExpectations ( );
		}

		[Test]
		public void GetPublished_WithValidParameters_ShouldInvokePostsPublishedRetrieved ( ) {
			bool actual = false;

			var stub = MockRepository.GenerateStub <IPostRepository> ( );
			var stubCategoryRepository = MockRepository.GenerateMock <ICategoryRepository> ( );
			IPostService sut = new PostService ( stubLogger, stub, stubCategoryRepository, stubCacheProvider );

			sut.PostsPublishedRetrieved += ( s , e ) => { actual = true; };

			sut.GetPublished ( 1 , 10 );

			actual.Should ( ).Be.True ( );
		}

		[Test]
		public void GetPublished_WithValidParameters_ShouldInvokePostsPublishedRetrieving ( ) {
			bool actual = false;

			var stub = MockRepository.GenerateStub <IPostRepository> ( );
			var stubCategoryRepository = MockRepository.GenerateMock <ICategoryRepository> ( );
			IPostService sut = new PostService ( stubLogger, stub, stubCategoryRepository, stubCacheProvider );

			sut.PostsPublishedRetrieving += ( s , e ) => {
			                                	actual = true;
			                                	e.Cancel = true;
			                                };

			sut.GetPublished ( 1 , 10 );

			actual.Should ( ).Be.True ( );
		}

		[Test]
		public void GetPublished_WithValidParameters_ShouldInvokePostsRetrievedByMonth ( ) {
			bool actual = false;

			var stub = MockRepository.GenerateStub <IPostRepository> ( );
			var stubCategoryRepository = MockRepository.GenerateMock <ICategoryRepository> ( );
			IPostService sut = new PostService ( stubLogger, stub, stubCategoryRepository, stubCacheProvider );

			sut.PostsRetrievedByMonth += ( s , e ) => { actual = true; };

			sut.GetByMonth ( 2 , 2010 , 1 , 10 );

			actual.Should ( ).Be.True ( );
		}

		[Test]
		public void GetPublished_WithValidParameters_ShouldInvokeRepositoryMethod ( ) {
			var mock = MockRepository.GenerateMock <IPostRepository> ( );
			var stubCategoryRepository = MockRepository.GenerateMock <ICategoryRepository> ( );
			IPostService sut = new PostService ( stubLogger, mock, stubCategoryRepository, stubCacheProvider );

			mock.Expect ( x => x.GetPublished ( 1 , 10 , false , false ) ).Return ( null );

			sut.GetPublished ( 1 , 10 );

			mock.VerifyAllExpectations ( );
		}

		[Test]
		public void GetRecent_WithValidParameters_ShouldInvokePostsRecentRetrieved ( ) {
			bool actual = false;

			var stub = MockRepository.GenerateStub <IPostRepository> ( );
			var stubCategoryRepository = MockRepository.GenerateMock <ICategoryRepository> ( );
			IPostService sut = new PostService ( stubLogger, stub, stubCategoryRepository, stubCacheProvider );

			sut.PostsRecentRetrieved += ( s , e ) => { actual = true; };

			sut.GetRecent ( 1 );

			actual.Should ( ).Be.True ( );
		}

		[Test]
		public void GetRecent_WithValidParameters_ShouldInvokePostsRecentRetrieving ( ) {
			bool actual = false;

			var stub = MockRepository.GenerateStub <IPostRepository> ( );
			var stubCategoryRepository = MockRepository.GenerateMock <ICategoryRepository> ( );
			IPostService sut = new PostService ( stubLogger, stub, stubCategoryRepository, stubCacheProvider );

			sut.PostsRecentRetrieving += ( s , e ) => {
			                             	actual = true;
			                             	e.Cancel = true;
			                             };

			sut.GetRecent ( 1 );

			actual.Should ( ).Be.True ( );
		}

		[Test]
		public void GetRecent_WithValidParameters_ShouldInvokeRepositoryMethod ( ) {
			var mock = MockRepository.GenerateMock <IPostRepository> ( );
			var stubCategoryRepository = MockRepository.GenerateMock <ICategoryRepository> ( );
			IPostService sut = new PostService ( stubLogger, mock, stubCategoryRepository, stubCacheProvider );

			mock.Expect ( x => x.GetRecent ( 1 , false , false ) ).Return ( null );

			sut.GetRecent ( 1 );

			mock.VerifyAllExpectations ( );
		}

		[Test]
		public void Save_WithValidParameters_ShouldInvokePostSaved ( ) {
			bool actual = false;

			var stub = MockRepository.GenerateStub <IPostRepository> ( );
			var stubCategoryRepository = MockRepository.GenerateMock <ICategoryRepository> ( );
			IPostService sut = new PostService ( stubLogger, stub, stubCategoryRepository, stubCacheProvider );

			sut.PostSaved += ( s , e ) => {
			                 	actual = true;
			                 	e.Objects.Should ( ).Be.EqualTo ( null );
			                 };

			sut.Save ( null );

			actual.Should ( ).Be.True ( );
		}

		[Test]
		public void Save_WithValidParameters_ShouldInvokePostSaving ( ) {
			bool actual = false;

			var stub = MockRepository.GenerateStub <IPostRepository> ( );
			var stubCategoryRepository = MockRepository.GenerateMock <ICategoryRepository> ( );
			IPostService sut = new PostService ( stubLogger, stub, stubCategoryRepository, stubCacheProvider );

			sut.PostSaving += ( s , e ) => {
			                  	actual = true;
			                  	e.Parameter.Should ( ).Be.EqualTo ( null );
			                  	e.Cancel = true;
			                  };

			sut.Save ( null );

			actual.Should ( ).Be.True ( );
		}

		[Test]
		public void Save_WithValidParameters_ShouldInvokeRepositoryMethod ( ) {
			var mock = MockRepository.GenerateMock <IPostRepository> ( );
			var stubCategoryRepository = MockRepository.GenerateMock <ICategoryRepository> ( );
			IPostService sut = new PostService ( stubLogger, mock, stubCategoryRepository, stubCacheProvider );

			mock.Expect ( x => x.Save ( null ) );

			sut.Save ( null );

			mock.VerifyAllExpectations ( );
		}

		[Test]
		public void Update_WithValidParameters_ShouldInvokePostSaved ( ) {
			bool actual = false;

			var stub = MockRepository.GenerateStub <IPostRepository> ( );
			var stubCategoryRepository = MockRepository.GenerateMock <ICategoryRepository> ( );
			IPostService sut = new PostService ( stubLogger, stub, stubCategoryRepository, stubCacheProvider );

			sut.PostUpdated += ( s , e ) => {
			                   	actual = true;
			                   	e.Objects.Should ( ).Be.EqualTo ( null );
			                   };

			sut.Update ( null );

			actual.Should ( ).Be.True ( );
		}

		[Test]
		public void Update_WithValidParameters_ShouldInvokePostSaving ( ) {
			bool actual = false;

			var stub = MockRepository.GenerateStub <IPostRepository> ( );
			var stubCategoryRepository = MockRepository.GenerateMock <ICategoryRepository> ( );
			IPostService sut = new PostService ( stubLogger, stub, stubCategoryRepository, stubCacheProvider );

			sut.PostUpdating += ( s , e ) => {
			                    	actual = true;
			                    	e.Parameter.Should ( ).Be.EqualTo ( null );
			                    	e.Cancel = true;
			                    };

			sut.Update ( null );

			actual.Should ( ).Be.True ( );
		}

		[Test]
		public void Update_WithValidParameters_ShouldInvokeRepositoryMethod ( ) {
			var mock = MockRepository.GenerateMock <IPostRepository> ( );
			var stubCategoryRepository = MockRepository.GenerateMock <ICategoryRepository> ( );
			IPostService sut = new PostService ( stubLogger, mock, stubCategoryRepository, stubCacheProvider );

			mock.Expect ( x => x.Update ( null ) );

			sut.Update ( null );

			mock.VerifyAllExpectations ( );
		}
	}
}