﻿ #region Disclaimer/Info
 
 /////////////////////////////////////////////////////////////////////////////////////////////////
 //
 //   File:		UserServiceTest.cs
 //   Website:		http://dexterblogengine.com/
 //   Authors:		http://dexterblogengine.com/About.ashx
 //   Rev:		1
 //   Created:		28/12/2010
 //   Last edit:		19/01/2011
 //   License:		GNU Library General Public License (LGPL)
 //   File:            UserServiceTest.cs
 //   For updated news and information please visit http://dexterblogengine.com/
 //   Dexter is hosted to Codeplex at http://dexterblogengine.codeplex.com
 //   For any question contact info@dexterblogengine.com
 //
 ///////////////////////////////////////////////////////////////////////////////////////////////////
 
 #endregion

using System.Net.Mail;
using Dexter.Dependency;
using Dexter.Domain.Model;
using Dexter.Logger.Concretes;
using Dexter.Logger.Contracts;
using Dexter.Repository;
using NUnit.Framework;
using Rhino.Mocks;
using SharpTestsEx;

namespace Dexter.Services.Implementation.Test {

	[TestFixture]
	public class UserServiceTest 
	{
		ILogger stubLogger;
		User testUser;

		[TestFixtureSetUp]
		public void FixtureSetUp ( ) {
			stubLogger = MockRepository.GenerateStub<ILogger> ( );

			IDexterContainer stubContainer = MockRepository.GenerateStub<IDexterContainer> ( );
			stubContainer.Stub ( x => x.Resolve<ILogger> ( ) ).Return ( NullLogger.Instance );
			DexterContainer.SetCurrent ( stubContainer );

			testUser = new User ( string.Empty, HashProvider.SHA1, string.Empty, EncryptDataFormat.Clear );
		}

		private IRoleRepository stubRole = MockRepository.GenerateStub<IRoleRepository>();

		#region GetUserByEmail Method

		[Test]
		public void GetUserByEmail_WithValidParameters_ShouldInvokeUserRetrievingByEmail ( ) {
			bool actual = false;
			var parameter = new MailAddress ( "yourname@yourdomain.com" );

			var stub = MockRepository.GenerateStub <IUserRepository> ( );
			IUserService sut = new UserService ( stubLogger, stub, stubRole );

			sut.UserRetrievingByEmail += ( s , e ) => {
			                             	actual = true;
			                             	e.Parameter.Should ( ).Be.EqualTo ( parameter );
			                             	e.Cancel = true;
			                             };

			sut.GetUserByEmail ( parameter );

			actual.Should ( ).Be.True ( );
		}

		[Test]
		public void GetUserByEmail_WithValidParameters_ShouldInvokeUserRetrievedByEmail()
		{
			bool actual = false;
			var parameter = new MailAddress("yourname@yourdomain.com");

			var stub = MockRepository.GenerateStub<IUserRepository>();
			IUserService sut = new UserService(stubLogger, stub, stubRole);

			sut.UserRetrievedByEmail += (s, e) =>
			                            {
			                            	actual = true;
			                            	e.Parameter.Should().Be.EqualTo(parameter);
			                            };

			sut.GetUserByEmail(parameter);

			actual.Should().Be.True();
		}

		[Test]
		public void GetUserByEmail_WithValidParameters_ShouldInvokeRepositoryMethod()
		{
			var parameter = new MailAddress("yourname@yourdomain.com");

			var mock = MockRepository.GenerateMock<IUserRepository>();
			IUserService sut = new UserService(stubLogger, mock, stubRole);

			mock.Expect(x => x.GetByEmailWithRoles(parameter)).Return(null);

			sut.GetUserByEmail(parameter);

			mock.VerifyAllExpectations (  );
		}

		#endregion

		#region GetUserByUsername Method

		[Test]
		public void GetUserByUsername_WithValidParameters_ShouldInvokeUserRetrievingByUsername ( ) {
			bool actual = false;
			var parameter = "imperugo";

			var stub = MockRepository.GenerateStub<IUserRepository> ( );
			IUserService sut = new UserService ( stubLogger, stub, stubRole );

			sut.UserRetrievingByUsername += ( s, e ) =>
			{
				actual = true;
				e.Parameter.Should ( ).Be.EqualTo ( parameter );
				e.Cancel = true;
			};

			sut.GetUserByUsername ( parameter );

			actual.Should ( ).Be.True ( );
		}

		[Test]
		public void GetUserByUsername_WithValidParameters_ShouldInvokeUserRetrievedByUsername ( ) {
			bool actual = false;
			var parameter = "imperugo";

			var stub = MockRepository.GenerateStub<IUserRepository> ( );
			IUserService sut = new UserService ( stubLogger, stub, stubRole );

			sut.UserRetrievedByUsername += ( s, e ) =>
			{
				actual = true;
				e.Parameter.Should ( ).Be.EqualTo ( parameter );
			};

			sut.GetUserByUsername ( parameter );

			actual.Should ( ).Be.True ( );
		}

		[Test]
		public void GetUserByUsername_WithValidParameters_ShouldInvokeRepositoryMethod ( ) {
			var parameter = "imperugo";

			var mock = MockRepository.GenerateMock<IUserRepository> ( );
			IUserService sut = new UserService ( stubLogger, mock, stubRole );

			mock.Expect ( x => x.GetByUsernameWithRoles ( parameter ) ).Return ( null );

			sut.GetUserByUsername ( parameter );

			mock.VerifyAllExpectations ( );
		}

		#endregion

		#region GetUserByKey Method

		[Test]
		public void GetUserByKey_WithValidParameters_ShouldInvokeUserRetrievingById ( ) {
			bool actual = false;
			var parameter = 1;

			var stub = MockRepository.GenerateStub<IUserRepository> ( );
			IUserService sut = new UserService ( stubLogger, stub, stubRole );

			sut.UserRetrievingById += ( s, e ) =>
			{
				actual = true;
				e.Parameter.Should ( ).Be.EqualTo ( parameter );
				e.Cancel = true;
			};

			sut.GetUserByKey ( parameter );

			actual.Should ( ).Be.True ( );
		}

		[Test]
		public void GetUserByKey_WithValidParameters_ShouldInvokeUserRetrievedById ( ) {
			bool actual = false;
			var parameter = 1;

			var stub = MockRepository.GenerateStub<IUserRepository> ( );
			IUserService sut = new UserService ( stubLogger, stub, stubRole );

			sut.UserRetrievedById += ( s, e ) =>
			{
				actual = true;
				e.Parameter.Should ( ).Be.EqualTo ( parameter );
			};

			sut.GetUserByKey ( parameter );

			actual.Should ( ).Be.True ( );
		}

		[Test]
		public void GetUserByKey_WithValidParameters_ShouldInvokeRepositoryMethod ( ) {
			var parameter = 1;

			var mock = MockRepository.GenerateMock<IUserRepository> ( );
			IUserService sut = new UserService ( stubLogger, mock, stubRole );

			mock.Expect ( x => x.GetByKeyWithRoles ( parameter ) ).Return ( null );

			sut.GetUserByKey ( parameter );

			mock.VerifyAllExpectations ( );
		}

		#endregion

		#region Save Method

		[Test]
		public void Save_WithValidParameters_ShouldInvokeUserCreating ( ) {
			bool actual = false;

			var stub = MockRepository.GenerateStub<IUserRepository> ( );
			IUserService sut = new UserService ( stubLogger, stub, stubRole );
			
			sut.UserCreating += ( s, e ) =>
			{
				actual = true;
				e.Parameter.Should().Be.EqualTo(testUser);
				e.Cancel = true;
			};

			sut.Save  ( testUser );

			actual.Should ( ).Be.True ( );
		}

		[Test]
		public void Save_WithValidParameters_ShouldInvokeUserCreated ( ) {
			bool actual = false;
			var stub = MockRepository.GenerateStub<IUserRepository> ( );
			IUserService sut = new UserService ( stubLogger, stub, stubRole );

			sut.UserCreated += ( s, e ) =>
			{
				actual = true;
			};

			sut.Save ( testUser );
			actual.Should ( ).Be.True ( );
		}

		[Test]
		public void Save_WithValidParameters_ShouldInvokeRepositoryMethod ( ) {
			var mock = MockRepository.GenerateMock<IUserRepository> ( );
			IUserService sut = new UserService ( stubLogger, mock, stubRole );

			mock.Expect ( x => x.Save ( testUser ) );

			sut.Save ( testUser );

			mock.VerifyAllExpectations ( );
		}

		#endregion

		#region Update Method

		[Test]
		public void Update_WithValidParameters_ShouldInvokeUserCreating ( ) {
			bool actual = false;

			var stub = MockRepository.GenerateStub<IUserRepository> ( );
			IUserService sut = new UserService ( stubLogger, stub, stubRole );

			sut.UserUpdating += ( s, e ) =>
			{
				actual = true;
				e.Parameter.Should ( ).Be.EqualTo ( null );
				e.Cancel = true;
			};

			sut.Update ( null );

			actual.Should ( ).Be.True ( );
		}

		[Test]
		public void Update_WithValidParameters_ShouldInvokeUserCreated ( ) {
			bool actual = false;
			var stub = MockRepository.GenerateStub<IUserRepository> ( );
			IUserService sut = new UserService ( stubLogger, stub, stubRole );

			sut.UserUpdated += ( s, e ) =>
			{
				actual = true;
			};

			sut.Update ( null );
			actual.Should ( ).Be.True ( );
		}

		[Test]
		public void Update_WithValidParameters_ShouldInvokeRepositoryMethod ( ) {
			var mock = MockRepository.GenerateMock<IUserRepository> ( );
			IUserService sut = new UserService ( stubLogger, mock, stubRole );

			mock.Expect ( x => x.Update ( null ) );

			sut.Update ( null );

			mock.VerifyAllExpectations ( );
		}

		#endregion

		#region Delete Method

		[Test]
		public void Delete_WithValidParameters_ShouldInvokeUserCreating ( ) {
			bool actual = false;

			var stub = MockRepository.GenerateStub<IUserRepository> ( );
			IUserService sut = new UserService ( stubLogger, stub, stubRole );

			sut.UserDeleting += ( s, e ) =>
			{
				actual = true;
				e.Parameter.Should ( ).Be.EqualTo ( null );
				e.Cancel = true;
			};

			sut.Delete ( null );

			actual.Should ( ).Be.True ( );
		}

		[Test]
		public void Delete_WithValidParameters_ShouldInvokeUserCreated ( ) {
			bool actual = false;
			var stub = MockRepository.GenerateStub<IUserRepository> ( );
			IUserService sut = new UserService ( stubLogger, stub, stubRole );

			sut.UserDeleted += ( s, e ) =>
			{
				actual = true;
			};

			sut.Delete ( null );

			actual.Should().Be.True (  );
		}

		[Test]
		public void Delete_WithValidParameters_ShouldInvokeRepositoryMethod ( ) {
			var mock = MockRepository.GenerateMock<IUserRepository> ( );
			IUserService sut = new UserService ( stubLogger, mock, stubRole );

			mock.Expect ( x => x.Delete ( null ) );

			sut.Delete ( null );

			mock.VerifyAllExpectations ( );
		}

		#endregion

		#region GetAllUsers Method

		[Test]
		public void GetAllUsers_WithValidParameters_ShouldInvokeUsersRetrieved ( ) {
			bool actual = false;
			var stub = MockRepository.GenerateStub<IUserRepository> ( );
			IUserService sut = new UserService ( stubLogger, stub, stubRole );

			sut.UsersRetrieved += ( s, e ) =>
			{
				actual = true;
			};

			sut.GetAllUsers ( 1,1 );

			actual.Should ( ).Be.True ( );
		}

		[Test]
		public void GetAllUsers_WithValidParameters_ShouldInvokeRepositoryMethod ( ) {
			var mock = MockRepository.GenerateMock<IUserRepository> ( );
			IUserService sut = new UserService ( stubLogger, mock, stubRole );

			mock.Expect ( x => x.GetListOfUserWithRoles ( 1,1 ) );

			sut.GetAllUsers ( 1, 1 );

			mock.VerifyAllExpectations ( );
		}

		#endregion

		#region FindUsersByName Method

		[Test]
		public void FindUsersByName_WithValidParameters_ShouldInvokeUsersRetrievedByName ( ) {
			bool actual = false;
			var stub = MockRepository.GenerateStub<IUserRepository> ( );
			IUserService sut = new UserService ( stubLogger, stub, stubRole );

			sut.UsersRetrievedByName += ( s, e ) =>
			{
				actual = true;
			};

			sut.FindUsersByName ( null, 1, 1 );

			actual.Should ( ).Be.True ( );
		}

		[Test]
		public void FindUsersByName_WithValidParameters_ShouldInvokeRepositoryMethod ( ) {
			var mock = MockRepository.GenerateMock<IUserRepository> ( );
			IUserService sut = new UserService ( stubLogger, mock, stubRole );

			mock.Expect ( x => x.FindUsersByNameWithRoles ( null, 1, 1 ) );

			sut.FindUsersByName (null, 1, 1 );

			mock.VerifyAllExpectations ( );
		}

		#endregion

		#region FindUsersByEmail Method

		[Test]
		public void FindUsersByEmail_WithValidParameters_ShouldInvokeUsersRetrievedByEmail ( ) {
			bool actual = false;
			var stub = MockRepository.GenerateStub<IUserRepository> ( );
			IUserService sut = new UserService ( stubLogger, stub, stubRole );

			sut.UsersRetrievedByEmail += ( s, e ) =>
			{
				actual = true;
			};

			sut.FindUsersByEmail ( null, 1, 1 );

			actual.Should ( ).Be.True ( );
		}

		[Test]
		public void FindUsersByEmail_WithValidParameters_ShouldInvokeRepositoryMethod ( ) {
			var mock = MockRepository.GenerateMock<IUserRepository> ( );
			IUserService sut = new UserService ( stubLogger, mock, stubRole );

			mock.Expect ( x => x.FindUsersByEmailWithRoles ( null, 1, 1 ) );

			sut.FindUsersByEmail ( null, 1, 1 );

			mock.VerifyAllExpectations ( );
		}

		#endregion
	}
}
